/*
 * File:     wx_dialg.h
 * Purpose:  wxDialogBox and common dialog declarations
 *
 *                       wxWindows 1.40
 * Copyright (c) 1993 Artificial Intelligence Applications Institute,
 *                   The University of Edinburgh
 *
 *                     Author: Julian Smart
 *                       Date: 18-4-93
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice, author statement and this permission
 * notice appear in all copies of this software and related documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS,
 * IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL THE ARTIFICIAL INTELLIGENCE APPLICATIONS INSTITUTE OR THE
 * UNIVERSITY OF EDINBURGH BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF
 * DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef wxb_dialgh
#define wxb_dialgh

#include "common.h"
#include "wx_panel.h"

#ifdef wx_curs
#define wxDIALOG_DEFAULT_X   20
#define wxDIALOG_DEFAULT_Y   5
#define wxDIALOG_DEFAULT_W   40
#define wxDIALOG_DEFAULT_H   12
#define wxCHOICE_HEIGHT 5
#define wxCHOICE_WIDTH 12
#else
#define wxDIALOG_DEFAULT_X  300
#define wxDIALOG_DEFAULT_Y  300
#define wxDIALOG_DEFAULT_W  300
#define wxDIALOG_DEFAULT_H  250
#define wxCHOICE_HEIGHT 150
#define wxCHOICE_WIDTH 200
#endif

// Dialog boxes
class wxbDialogBox: public wxPanel
{
 public:
  Bool modal;

  wxbDialogBox(void);
  wxbDialogBox(wxFrame *frame, char *title, Bool modal = FALSE,
              int x = -1, int y = -1,
	      int width = -1, int height = -1, int style = 0, Bool create = FALSE);

  virtual void Show(Bool show) = 0;
  virtual void Centre(int direction = wxBOTH);
  virtual char *GetTitle(void) = 0;
  virtual void SetTitle(char *title) = 0;
  virtual void Fit(void) = 0;
  inline char *GetLabel(void) { return GetTitle(); }
};

extern int wxDialogButtonPressed;
void wxDialogOkButton(wxButton& but, wxEvent& event);
void wxDialogCancelButton(wxButton& but, wxEvent& event);
void wxDialogYesButton(wxButton& but, wxEvent& event);
void wxDialogNoButton(wxButton& but, wxEvent& event);

// Handy dialog functions
char *wxGetTextFromUser(char *message, char *caption = "Input text",
                        char *default_value = "", wxFrame *parent = NULL,
                        int x = -1, int y = -1, Bool centre = TRUE);

char *wxGetSingleChoice(char *message, char *caption,
                        int n, char *choices[], wxFrame *parent = NULL,
                        int x = -1, int y = -1, Bool centre = TRUE,
                        int width = wxCHOICE_WIDTH, int height = wxCHOICE_HEIGHT);

// Same as above but gets position in list of strings, instead of string,
// or -1 if no selection
int wxGetSingleChoiceIndex(char *message, char *caption,
			   int n, char *choices[], wxFrame *parent = NULL,
			   int x = -1, int y = -1, Bool centre = TRUE,
			   int width = wxCHOICE_WIDTH, int height = wxCHOICE_HEIGHT);

// Return client data instead
char *wxGetSingleChoiceData(char *message, char *caption,
                            int n, char *choices[], char *client_data[],
                            wxFrame *parent = NULL, int x = -1, int y = -1,
                            Bool centre = TRUE,
                            int width = wxCHOICE_WIDTH, int height = wxCHOICE_HEIGHT);
                           
// Returns directly after the selection is made
char *wxGetSinglePopUp(char *message, char *caption,
			int n, char *choices[], wxFrame *parent = NULL,
			int x = -1, int y = -1, Bool centre = TRUE,
			int width = -1, int height = -1);

int wxGetSinglePopUpIndex(char *message, char *caption,
			   int n, char *choices[], wxFrame *parent = NULL,
			   int x = -1, int y = -1, Bool centre = TRUE,
			   int width = -1, int height = -1);

// type is an 'or' (|) of wxOK, wxCANCEL, wxYES_NO
// Returns wxYES/NO/OK/CANCEL
int wxMessageBox(char *message, char *caption = "Message", int type = wxOK,
  wxFrame *parent = NULL, int x = -1, int y = -1, Bool centre = TRUE);

#define wxOPEN 1
#define wxSAVE 2
#define wxOVERWRITE_PROMPT 4
#define wxHIDE_READONLY 8

// File selector
char *wxFileSelector(char *message = "Select a file", char *default_path = NULL,
                     char *default_filename = NULL, char *default_extension = NULL,
                     char *wildcard = "*.*", int flags = 0,
                     wxFrame *parent = NULL, int x = -1, int y = -1);

#endif // wxb_dialgh
