#ifndef _curswin_h
#define _curswin_h

#include <stdarg.h>
#include "wx_list.h"
#include "wx_obj.h"
#include "wx_rect.h"
#include "common.h"

#define CW_NO_LINE 0
#define CW_THICK_LINE 1
#define CW_THIN_LINE 2
#define CW_BOLD_LINE 16

#define CUR_HIDE 0
#define CUR_NORM 1
#define CUR_INSERT 2

#undef KEY_TAB
#define KEY_TAB 9
#undef KEY_ESC
#define KEY_ESC 27
#undef KEY_ENTER
#define KEY_ENTER 13
#define KEY_SPACE 32
#ifndef KEY_DEL
#define KEY_DEL 0177
#endif
#undef  CNTRL
#define CNTRL(c)	c - 'A' + 1

#define MOD_CTRL  1
#define MOD_SHIFT 2
#define MOD_META  3

class wxWindow;

class CursWnd : public wxObject
{
public: 
  WINDOW *Handle;
  Bool Visible;
  int BorderStyle;
  int CursorStyle;
  int WinAttribs;
  CursWnd *Parent;
  wxList Children;
  wxRect WinRect;
  wxWindow* wx_window;

  CursWnd(void);
  void InitWin(void);
  void AddChild(CursWnd *child);
  void RemoveChild(CursWnd *child);
  void RemoveChildren(void);
  Bool NeedUpdate(CursWnd *wnd);
  wxRect ScreenRect(void);
  void ValidateWinRect(wxRect& rect);
  void Move(int dx, int dy);
  void Resize(int nw, int nh);
  void Refresh(int sy, int by);
 
  CursWnd(CursWnd *parent, wxRect& winRect, int bstyle, wxWindow *win);
  ~CursWnd();

  void BringToTop(void);		// make active and visible
  void SetFocus(CursWnd *wnd);
  void SetCursor(int type);
  wxRect GetWinRect(void);
  wxRect GetClientRect(void);
  void SetWinRect(wxRect& winRect);
  void GetPos(int *x, int *y);			// get pos
  void SetPos(int x, int y);			// move the window
  void GetSize(int *w, int *h);			// get size
  void GetClientSize(int *w, int *h);
  void SetSize(int w, int h);			// resize the window 

  void Clear(void);				// clears the window
  void Scroll(int nolines);			// Scroll up/down
  void Border(int type = CW_THICK_LINE);	// draw thick or thin border
  void Show(Bool show);			// hide or show the window

  int WinAttrib(void);
  void WinAttrib(int winattrib);
  void WinAttribOn(int winattrib);
  void WinAttribOff(int winattrib);

  void GotoXY(int x, int y);			// moves the cursor
  void GetXY(int *x, int *y);			// get cursor pos
  void CurLeft(void);			// cursor movement
  void CurRight(void);
  void CurUp(void);
  void CurDown(void);

  void Print(char c);
  void Print(char *s);
  void Printf(char *fmt, ...);
  void Print(int x, int y, char c);
  void Print(int x, int y, char* s);
  void Printf(int x, int y, char *fmt, ...);
  void Print(int style, char c);
  void Print(int style, char *s);
  void Printf(int style, char *fmt, ...);
  void Print(int style, int x, int y, char c);
  void Print(int style, int x, int y, char* s);
  void Printf(int style, int x, int y, char *fmt, ...);

// Non Clipping version without checking
  void PrintNC(char c);
  void PrintNC(char* s);
  void PrintfNC(char *fmt, ...);
  void PrintNC(int x, int y, char c);
  void PrintNC(int x, int y, char* s);
  void PrintfNC(int x, int y, char *fmt, ...);
  void PrintNC(int style, char c);
  void PrintNC(int style, char* s);
  void PrintfNC(int style, char *fmt, ...);
  void PrintNC(int style, int x, int y, char c);
  void PrintNC(int style, int x, int y, char* s);
  void PrintfNC(int style, int x, int y, char *fmt, ...);

};

class ScrWnd : public CursWnd
{
  WINDOW *StatusLine;
  Bool DoUpdate;

public:
   wxRect UpdateReg;

  ScrWnd(void);
  ~ScrWnd();

  void InvalidateRect(wxRect rect);
  void Refresh(int sy, int count);
  void Refresh(void);
  void Clear(void);
  void RefreshStatusLine(void);
  void RefreshCursor(void);
  void CreateStatusLine(void);
  void StatusMsg(char *fmt, ...);
  void StatusMsg(int style, char *fmt, ...);
  Bool KeyAvail(void);
  int GetKey(void);
  int WaitKey(void);
  int PeekKey(void);
  void SetCursor(int type = CUR_NORM);	// hide, normal or insert mode
  void Update(Bool yn);
};

extern ScrWnd* TheScreen;
extern wxWindow* FocusWnd;
extern wxWindow* FocusRootWnd;

#endif
