/*
 * File:     wx_gdi.h
 * Purpose:  Declaration of various graphics objects - fonts, pens, icons etc.
 *
 *                       wxWindows 1.40
 * Copyright (c) 1993 Artificial Intelligence Applications Institute,
 *                   The University of Edinburgh
 *
 *                     Author: Julian Smart
 *                       Date: 18-4-93
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice, author statement and this permission
 * notice appear in all copies of this software and related documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS,
 * IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL THE ARTIFICIAL INTELLIGENCE APPLICATIONS INSTITUTE OR THE
 * UNIVERSITY OF EDINBURGH BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF
 * DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#ifndef wx_gdih
#define wx_gdih

#include "wb_gdi.h"

// Font
class wxFont: public wxbFont
{
 public:
  int curs_attribs;

  wxFont(void);
  wxFont(int PointSize, int Family, int Style, int Weight);
  ~wxFont(void);
};

class wxColourMap: public wxbColourMap
{
 public:
  wxColourMap(void);
  ~wxColourMap(void);
};

#define wxColorMap wxColourMap

// Pen
class wxPen: public wxbPen
{
 public:
  wxPen(void);
  wxPen(int width);
  wxPen(wxColour& col, int width, int style);
  wxPen(char *col, int width, int style);
  ~wxPen(void);

  virtual void SetColour(wxColour& col);
  virtual void SetColour(char *col);
  virtual void SetColour(char r, char g, char b);

  virtual void SetWidth(int width);
  virtual void SetStyle(int style);
};


// Brush
class wxBrush: public wxbBrush
{
 public:
  wxBrush(void);
  wxBrush(wxColour& col, int style);
  wxBrush(char *col, int style);
  ~wxBrush(void);

  virtual void SetColour(wxColour& col);
  virtual void SetColour(char *col);
  virtual void SetColour(char r, char g, char b);

  virtual void SetStyle(int style);
};

// Icon
class wxIcon: public wxbIcon
{
 public:

  wxIcon(char bits[], int width, int height);
  wxIcon(char *name);
  ~wxIcon(void);
};

// Cursor
class wxCursor: public wxbCursor
{
 public:
  wxCursor(char bits[], int width, int height);
  wxCursor(char *name);
  wxCursor(int cursor_type);
  ~wxCursor(void);
};

// Bitmap
class wxBitmap: public wxbBitmap
{
 public:

  wxBitmap(char bits[], int width, int height, int no_bits = 1);
  wxBitmap(char *name);
  ~wxBitmap(void);
};

#endif // wx_gdih
