/*
 * File:     wx_lbox.h
 * Purpose:  List box panel item
 *
 *                       wxWindows 1.40
 * Copyright (c) 1993 Artificial Intelligence Applications Institute,
 *                   The University of Edinburgh
 *
 *                     Author: Julian Smart
 *                       Date: 18-4-93
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice, author statement and this permission
 * notice appear in all copies of this software and related documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS,
 * IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL THE ARTIFICIAL INTELLIGENCE APPLICATIONS INSTITUTE OR THE
 * UNIVERSITY OF EDINBURGH BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF
 * DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef wx_lboxh
#define wx_lboxh

#include "wb_lbox.h"

// List box item
class wxListBox: public wxbListBox
{
protected:
  CursWnd *static_label;
  wxList *listItems;
  int no_selected;
  int view_start, view_pos;

public:
  wxListBox(wxPanel *panel, wxFunction func, char *Title, Bool Multiple = FALSE,
             int x = -1, int y = -1, int width = -1, int height = -1,
	     int N = 0, char **Choices = NULL, Bool create = TRUE);
  Bool Create(wxPanel *panel, wxFunction func, char *Title, Bool Multiple = FALSE,
             int x = -1, int y = -1, int width = -1, int height = -1,
             int N = 0, char **Choices = NULL);
  ~wxListBox(void);


  void Append(char *Item);
  void Append(char *Item, char *Client_data);
  void Set(int N, char *Choices[]);
  int FindString(char *s);
  void Clear(void);
  void SetSelection(int N);

  void Deselect(int N);

  // For single choice list item only
  int GetSelection(void);
  void Delete(int N);
  char *GetClientData(int N);
  // For single or multiple choice list item
  int GetSelections(int **list_selections);
  char *GetString(int N);
  void OnPaint(void);
  void GetPosition(int *x, int *y);
  void GetSize(int *width, int *height);
  void GetClientSize(int *width, int *height);
  void SetSize(int x, int y, int width, int height);
  void Show(Bool flag);

  virtual void OnSelectionFocus(int selfocus);
  void OnSetFocus(void);
  void OnKillFocus(void);
  int FindMaxSize(void);

  Bool ProcessEvent(wxEvent *ev);
  void HandleScrollUp(void);
  void HandleScrollDown(void);
  void HandleSelect(void);
};

#endif // wx_lboxh
