/*
 * File:     wx_menu.h
 * Purpose:  Declares panel items (controls/widgets)
 *
 *                       wxWindows 1.40
 * Copyright (c) 1993 Artificial Intelligence Applications Institute,
 *                   The University of Edinburgh
 *
 *                     Author: Julian Smart
 *                       Date: 18-4-93
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice, author statement and this permission
 * notice appear in all copies of this software and related documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS,
 * IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL THE ARTIFICIAL INTELLIGENCE APPLICATIONS INSTITUTE OR THE
 * UNIVERSITY OF EDINBURGH BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF
 * DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef wx_menuh
#define wx_menuh

#include "common.h"
#include "wx_win.h"
#include "wx_panel.h"
#include "wb_menu.h"

class wxMenuBar;

// Menu
class wxMenu: public wxbMenu
{
 public:
  wxPanel *menu_panel;

  wxMenu(char *Title = NULL, wxFunction func = NULL);
  Bool Create(wxFrame *parent, int x, int y, int width, int height, int labelPos);
  ~wxMenu(void);
  void AppendSeparator(void);
  void Append(int Id, char *Label, char *helpString = NULL);
  void Append(int Id, char *Label, wxMenu *SubMenu, char *helpString = NULL);
  void Enable(int Id, Bool Flag);
  void Check(int Id, Bool Flag);

  void Show(Bool flag);
  void SetFocus(void);
  void SetSize(int x, int y, int width, int height);
  wxRect MinimalWinSize(void);
  void Hide(void);
  void HideAll(void);
  int FindMaxSize(void);
};

class wxFrame;
class wxMenuBar:public wxbMenuBar
{
 public:
  wxMenu *menu_bar_menu;

  wxMenuBar(void);
  wxMenuBar(int n, wxMenu *menus[], char *Titles[]);
  Bool Create(wxFrame *parent, int x, int y, int width, int height);
  ~wxMenuBar(void);

  void SetFocus(void);
  wxRect MinimalWinSize(void);
  void SetSize(int x, int y, int width, int height);

  void Append(wxMenu *menu, char *title);
  // Must only be used AFTER menu has been attached to frame,
  // otherwise use individual menus to enable/disable items
  void Enable(int Id, Bool Flag);
  void Check(int Id, Bool Flag);
};
#endif // wx_menuh
