/*
 * File:     wx_win.h
 * Purpose:  wxWindow class declaration. Base class for all windows and
 *           panel items.
 *
 *                       wxWindows 1.40
 * Copyright (c) 1993 Artificial Intelligence Applications Institute,
 *                   The University of Edinburgh
 *
 *                     Author: Julian Smart
 *                       Date: 18-4-93
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice, author statement and this permission
 * notice appear in all copies of this software and related documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS,
 * IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL THE ARTIFICIAL INTELLIGENCE APPLICATIONS INSTITUTE OR THE
 * UNIVERSITY OF EDINBURGH BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF
 * DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef wx_winh
#define wx_winh

#include "wb_win.h"
#include "curswin.h"
#include "wx_rect.h"

/*
 * Base class for frame, panel, canvas, panel items, dialog box.
 *
 */
class wxFrame;

extern wxWindow *wxFocusWnd;
extern wxFrame *wxFocusFrame;

class wxWindow: public wxbWindow
{
 public:
  CursWnd* handle;
  Bool wantsFocus;
  wxList *disposal;
  wxList *preEventHandlers, *postEventHandlers;

  // Internal member functions
  void AddDisposal(wxObject *disposable);
  void RemoveDisposal(void);
  virtual wxRect GetWinRect(void);
  virtual wxRect MinimalWinSize(void);
  virtual wxRect GetClientRect(void);
  virtual void ValidateWinRect(wxRect& rect);
  wxWindow* FocusedSubWin(void);
  virtual Bool WantFocus(void);
  virtual void WantFocus(Bool flag);
  virtual Bool InFocusChain(void);
  virtual void FocusPreviousWin(void);
  virtual void FocusPreviousFrame(void);
  virtual void FocusNextSubWin(void);
  virtual void FocusPreviousSubWin(void);
  virtual void KillFocus(void);
  virtual Bool Visible(void);
  virtual void ScreenToClient(wxRect& rect);
  virtual void ClientToScreen(wxRect& rect);
  virtual void Refresh(void);
  virtual void WantPreProcessEvent(wxWindow *win);
  virtual void WantPostProcessEvent(wxWindow *win);
  virtual Bool PreProcessEvent(wxEvent *event);
  virtual Bool ProcessEvent(wxEvent *event);
  virtual Bool PostProcessEvent(wxEvent *event, Bool handled);

  // Constructors/Destructors
  wxWindow(void);
  virtual ~wxWindow(void);

  char *GetHandle(void);
  virtual int XChar(int pixels);              // Convert pixels to character positions
  virtual int YChar(int pixels);

  virtual void Show(Bool show);
  virtual wxCursor *SetCursor(wxCursor *cursor);
  virtual void SetColourMap(wxColourMap *cmap);

  virtual int GetTextFamily(void);
  virtual int GetTextStyle(void);
  virtual int GetTextWeight(void);
  virtual float GetTextHeight(void);
  virtual float GetTextWidth(void);
  virtual void GetTextExtent(char *string, float *x, float *y);

  virtual void GetSize(int *width, int *height);
  virtual void GetPosition(int *x, int *y);
  virtual void GetClientSize(int *width, int *height); // Size client can use
  virtual void SetSize(int x, int y, int width, int height);
  virtual void SetClientSize(int width, int size);
  virtual void SetFocus(void);

  virtual Bool OnHelp(void);

  void Enable(Bool enable) ;
  void CaptureMouse(void);
  void ReleaseMouse(void);
  void DragAcceptFiles(Bool accept);

};

#endif
