/*
 * File:     minimal.cc
 * Purpose:  Minimal wxWindows app
 *
 *                       wxWindows 1.40
 * Copyright (c) 1993 Artificial Intelligence Applications Institute,
 *                   The University of Edinburgh
 *
 *                     Author: Julian Smart
 *                        Date: 18-4-93
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice, author statement and this permission
 * notice appear in all copies of this software and related documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS,
 * IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL THE ARTIFICIAL INTELLIGENCE APPLICATIONS INSTITUTE OR THE
 * UNIVERSITY OF EDINBURGH BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF
 * DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "wx.h"

// Define a new application type
class MyApp: public wxApp
{ public:
    wxFrame *OnInit(void);
};

// Define a new frame type
class MyFrame: public wxFrame
{ public:
	MyFrame(wxFrame *frame, char *title, int x, int y, int w, int h);
        void OnMenuCommand(int id);
};


// This statement initializes the whole application and calls OnInit
MyApp myApp;


// `Main program' equivalent, creating windows and returning main app frame
wxFrame *MyApp::OnInit(void)
{
  // Create the main frame window
  MyFrame *frame = new MyFrame(NULL, "Minimal wxWindows App", 1, 1, 50, 20);
  frame->CreateStatusLine();

	// Create a second frame window
  wxFrame *frame2 = new MyFrame(NULL, "Second Frame", 10, 0, 65, 24);

  wxMenu *file_menu = new wxMenu;
  file_menu->Append(1, "Open...", "Help on Open");
  file_menu->Append(2, "Close", "Help on Close");
  file_menu->Append(3, "Quit", "Help on Quit");
  wxMenu *set_menu = new wxMenu;
  set_menu->Append(31, "Edit Settings...");
  set_menu->AppendSeparator();

  wxMenu *sub_menu = new wxMenu;
  sub_menu->Append(40, "Try me");
  sub_menu->Append(41, "En(dis)able 'Try me'");
  sub_menu->Append(42, "(Un)Check");
  set_menu->Append(32, "Sub Menu 1", sub_menu);

  wxMenuBar *menu_bar = new wxMenuBar;
  menu_bar->Append(file_menu, "File");
  menu_bar->Append(set_menu, "Settings");
  frame->SetMenuBar(menu_bar);

  menu_bar->Enable(40, FALSE);
  menu_bar->Check(42, TRUE);

  int width, height;
  frame->GetClientSize(&width, &height);
  wxTextWindow *txtwin = new wxTextWindow(frame, 0,0,width,height);
  txtwin->LoadFile("sample.txt");

  wxPanel *apanel = new wxPanel(frame2, -1,-1,-1,-1, wxBORDER);
  wxButton *but1 = new wxButton(apanel, NULL, "But 1");
  wxButton *but2 = new wxButton(apanel, NULL, "But 2");
  apanel->NewLine();
  wxCheckBox *check1 = new wxCheckBox(apanel, NULL, "Check 1");
  wxCheckBox *check2 = new wxCheckBox(apanel, NULL, "Check 2");
  apanel->NewLine();
  wxListBox *listb1 = new wxListBox(apanel, NULL, "Listbox (single):", wxSINGLE, -1, -1, -1, 4);
  char *listi1[] = {"hallo", "dag"};
  listb1->Set(2, listi1);
  wxListBox *listb2 = new wxListBox(apanel, NULL, "Listbox (multiple):", wxMULTIPLE, -1, -1, -1, 4);
  char *listi2[] = {"hello", "world", "how", "are", "you"};
  listb2->Set(5, listi2);
  apanel->NewLine();
  wxMessage *msg1 = new wxMessage(apanel, "Hello World 1");
  wxMessage *msg2 = new wxMessage(apanel, "Hello World 2");
  apanel->NewLine();
  wxChoice *choice1 = new wxChoice(apanel, NULL, "Choice 1", -1,-1,-1,-1, 2, listi1);
  wxChoice *choice2 = new wxChoice(apanel, NULL, "Choice 2", -1,-1,-1,-1, 5, listi2);
  apanel->NewLine();
  wxText *text1 = new wxText(apanel, NULL, "Text 1", "some text with a lot of space needed");
  apanel->NewLine();
  wxSlider *slid1 = new wxSlider(apanel, NULL, "Slider 1:", 5, 1, 10);
  wxMultiText *mtxt1 = new wxMultiText(apanel, NULL, "Multi:", "a line\nwith three\nlines");

  apanel->Show(TRUE);

  // Show the frame
  frame2->Show(TRUE);
  frame->Show(TRUE);

  // Return the main frame window
  return frame;
}

// My frame constructor
MyFrame::MyFrame(wxFrame *frame, char *title, int x, int y, int w, int h):
  wxFrame(frame, title, x, y, w, h)
{}

void MyFrame::OnMenuCommand(int id)
{ static Bool checked = TRUE;
  static Bool enabled = FALSE;
  switch(id)
  { case 1: 	char *filename = wxFileSelector("Open a file:");
		sprintf(wxBuffer, "Returned filename is:\n%s", filename);
		wxMessageBox(wxBuffer);
		break;
    case 3:     delete this;
		break;
    case 31:	wxMessageBox("Option not yet implemented", "Sorry");
		break;
    case 41:	enabled = !enabled;
		GetMenuBar()->Enable(40, enabled);
		break;
    case 42:	checked = !checked;
		GetMenuBar()->Check(42, checked);
		break;
    default:	wxMessageBox("Option not yet implemented", "Sorry");
                break;
  };
}


