/*
 * File:     wx_gdi.cc
 * Purpose:  GDI (Graphics Device Interface) objects and functions
 *
 *                       wxWindows 1.40
 * Copyright (c) 1993 Artificial Intelligence Applications Institute,
 *                   The University of Edinburgh
 *
 *                     Author: Julian Smart
 *                       Date: 18-4-93
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice, author statement and this permission
 * notice appear in all copies of this software and related documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS,
 * IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL THE ARTIFICIAL INTELLIGENCE APPLICATIONS INSTITUTE OR THE
 * UNIVERSITY OF EDINBURGH BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF
 * DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifdef wx_msw
#include "wx.h"
#pragma hdrstop
#else

#include "common.h"
#include "wx_list.h"
#include "wx_utils.h"
#include "wx_main.h"
#include "wx_gdi.h"
#endif

wxBrushList *wxTheBrushList = NULL;
wxPenList   *wxThePenList = NULL;
wxGDIList   *wxTheIconList = NULL;
wxGDIList   *wxTheFontList = NULL;
wxGDIList   *wxTheBitmapList = NULL;

wxColourDatabase *wxTheColourDatabase = NULL;

// Stock objects
wxFont *wxNORMAL_FONT;
wxFont *wxSMALL_FONT;
wxFont *wxITALIC_FONT;
wxFont *wxSWISS_FONT;
wxPen *wxRED_PEN;
wxPen *wxCYAN_PEN;
wxPen *wxGREEN_PEN;
wxPen *wxBLACK_PEN;
wxPen *wxTRANSPARENT_PEN;
wxPen *wxBLACK_DASHED_PEN;

wxBrush *wxBLUE_BRUSH;
wxBrush *wxGREEN_BRUSH;
wxBrush *wxWHITE_BRUSH;
wxBrush *wxBLACK_BRUSH;
wxBrush *wxTRANSPARENT_BRUSH;
wxBrush *wxCYAN_BRUSH;
wxBrush *wxRED_BRUSH;
wxBrush *wxGREY_BRUSH;
wxBrush *wxMEDIUM_GREY_BRUSH;
wxBrush *wxLIGHT_GREY_BRUSH;

wxColour *wxBLACK;
wxColour *wxWHITE;
wxColour *wxRED;
wxColour *wxBLUE;
wxColour *wxGREEN;
wxColour *wxCYAN;
wxColour *wxLIGHT_GREY;

wxCursor *wxSTANDARD_CURSOR = NULL;
wxCursor *wxHOURGLASS_CURSOR = NULL;
wxCursor *wxCROSS_CURSOR = NULL;

wxbFont::wxbFont(void)
{
  __type = wxTYPE_FONT;
}

/* Constructor for a font. Note that the real construction is done
 * in wxDC::SetFont, when information is available about scaling etc.
 */
wxbFont::wxbFont(int PointSize, int Family, int Style, int Weight, Bool Underline)
{
  __type = wxTYPE_FONT;
}

wxbFont::~wxbFont()
{
}

// Colour

wxColour::wxColour(char r, char g, char b)
{
  __type = wxTYPE_COLOUR;
  red = r; green = g; blue = b;
}

wxColour::wxColour(char *col)
{
  __type = wxTYPE_COLOUR;
  wxColour *the_colour = wxTheColourDatabase->FindColour(col);
  if (the_colour)
  {
    red = the_colour->Red(); green = the_colour->Green();
    blue = the_colour->Blue();
  }
  else
  {
    red = 0; green = 0; blue = 0;
  }
}


void wxColour::Set(char r, char g, char b)
{
  red = r; green = g; blue = b;
}

void wxColour::Get(char *r, char *g, char *b)
{
  *r = red; *g = green; *b = blue;
}

char wxColour::Red(void)
{
  return red;
}

char wxColour::Green(void)
{
  return green;
}

char wxColour::Blue(void)
{
  return blue;
}

/*
 * Colour map
 *
 */

wxbColourMap::wxbColourMap(void)
{
}

wxbColourMap::~wxbColourMap(void)
{
}

wxColourDatabase::wxColourDatabase(int type):wxList(type)
{
}

// Colour database stuff
void wxColourDatabase::Initialize(void)
{
  Append("AQUAMARINE"       , new wxColour(112, 219, 147));
  Append("BLACK"            , new wxColour(0, 0, 0));
  Append("BLUE"             , new wxColour(0, 0, 255));
  Append("BLUE VIOLET"      , new wxColour(159, 95, 159));
  Append("BROWN"            , new wxColour(165, 42, 42));
  Append("CADET BLUE"       , new wxColour(95, 159, 159));
  Append("CORAL"            , new wxColour(255, 127, 0));
  Append("CORNFLOWER BLUE"  , new wxColour(66, 66, 111));
  Append("CYAN"             , new wxColour(0, 255, 255));
  Append("DARK GREY"        , new wxColour(47, 47, 47));  // ?
  Append("DARK GREEN"       , new wxColour(47, 79, 47));
  Append("DARK OLIVE GREEN" , new wxColour(79, 79, 47));
  Append("DARK ORCHID"      , new wxColour(153, 50, 204));
  Append("DARK SLATE BLUE"  , new wxColour(107, 35, 142));
  Append("DARK SLATE GREY"  , new wxColour(47, 79, 79));
  Append("DARK TURQUOISE"   , new wxColour(112, 147, 219));
  Append("DIM GREY"         , new wxColour(84, 84, 84));
  Append("FIREBRICK"        , new wxColour(142, 35, 35));
  Append("FOREST GREEN"     , new wxColour(35, 142, 35));
  Append("GOLD"             , new wxColour(204, 127, 50));
  Append("GOLDENROD"        , new wxColour(219, 219, 112));
  Append("GREY"             , new wxColour(192, 192, 192));
  Append("GREEN"            , new wxColour(0, 255, 0));
  Append("GREEN YELLOW"     , new wxColour(147, 219, 112));
  Append("INDIAN RED"       , new wxColour(79, 47, 47));
  Append("KHAKI"            , new wxColour(159, 159, 95));
  Append("LIGHT BLUE"       , new wxColour(191, 216, 216));
  Append("LIGHT GREY"       , new wxColour(168, 168, 168));
//  Append("LIGHT GREY"       , new wxColour(200, 200, 200));
  Append("LIGHT STEEL BLUE" , new wxColour(143, 143, 188));
  Append("LIME GREEN"       , new wxColour(50, 204, 50));
  Append("MAGENTA"          , new wxColour(255, 0, 255));
  Append("MAROON"           , new wxColour(142, 35, 107));
  Append("MEDIUM AQUAMARINE" , new wxColour(50, 204, 153));
  Append("MEDIUM GREY" , new wxColour(100, 100, 100));
  Append("MEDIUM BLUE"      , new wxColour(50, 50, 204));
  Append("MEDIUM FOREST GREEN" , new wxColour(107, 142, 35));
  Append("MEDIUM GOLDENROD" , new wxColour(234, 234, 173));
  Append("MEDIUM ORCHID"    , new wxColour(147, 112, 219));
  Append("MEDIUM SEA GREEN" , new wxColour(66, 111, 66));
  Append("MEDIUM SLATE BLUE" , new wxColour(127, 0, 255));
  Append("MEDIUM SPRING GREEN" , new wxColour(127, 255, 0));
  Append("MEDIUM TURQUOISE" , new wxColour(112, 219, 219));
  Append("MEDIUM VIOLET RED" , new wxColour(219, 112, 147));
  Append("MIDNIGHT BLUE"    , new wxColour(47, 47, 79));
  Append("NAVY"             , new wxColour(35, 35, 142));
  Append("ORANGE"           , new wxColour(204, 50, 50));
  Append("ORANGE RED"       , new wxColour(255, 0, 127));
  Append("ORCHID"           , new wxColour(219, 112, 219));
  Append("PALE GREEN"       , new wxColour(143, 188, 143));
  Append("PINK"             , new wxColour(188, 143, 234));
  Append("PLUM"             , new wxColour(234, 173, 234));
  Append("PURPLE"           , new wxColour(176, 0, 255));
  Append("RED"              , new wxColour(255, 0, 0));
  Append("SALMON"           , new wxColour(111, 66, 66));
  Append("SEA GREEN"        , new wxColour(35, 142, 107));
  Append("SIENNA"           , new wxColour(142, 107, 35));
  Append("SKY BLUE"         , new wxColour(50, 153, 204));
  Append("SLATE BLUE"       , new wxColour(0, 127, 255));
  Append("SPRING GREEN"     , new wxColour(0, 255, 127));
  Append("STEEL BLUE"       , new wxColour(35, 107, 142));
  Append("TAN"              , new wxColour(219, 147, 112));
  Append("THISTLE"          , new wxColour(216, 191, 216));
  Append("TURQUOISE"        , new wxColour(173, 234, 234));
  Append("VIOLET"           , new wxColour(79, 47, 79));
  Append("VIOLET RED"       , new wxColour(204, 50, 153));
  Append("WHEAT"            , new wxColour(216, 216, 191));
  Append("WHITE"            , new wxColour(255, 255, 255));
  Append("YELLOW"           , new wxColour(255, 255, 0));
  Append("YELLOW GREEN"     , new wxColour(153, 204, 50));
}

wxColour *wxColourDatabase::FindColour(char *colour)
{
  wxNode *node = Find(colour);
  if (node)
    return (wxColour *)node->Data();
  else
    return NULL;
}

char *wxColourDatabase::FindName(wxColour& colour)
{
  wxNode *node = First();
  char *found = NULL;
  char red = colour.Red();
  char green = colour.Green();
  char blue = colour.Blue();
  while (node && !found)
  {
    wxColour *col = (wxColour *)node->Data();
    if (col->Red() == red && col->Green() == green && col->Blue() == blue)
      found = node->key.string;
    else
      node = node->Next();
  }
  return found;
}


void wxInitializeStockObjects(void)
{
  wxTheBrushList = new wxBrushList;
  wxThePenList = new wxPenList;
  wxTheIconList = new wxGDIList;
  wxTheFontList =  new wxGDIList;
  wxTheBitmapList =  new wxGDIList;

#ifdef wx_motif
#endif
#ifdef wx_x
  wxFontPool = new XFontPool;
#endif

  wxNORMAL_FONT = new wxFont(12, wxMODERN, wxNORMAL, wxNORMAL);
  wxSMALL_FONT = new wxFont(10, wxSWISS, wxNORMAL, wxNORMAL);
  wxITALIC_FONT = new wxFont(12, wxROMAN, wxITALIC, wxNORMAL);
  wxSWISS_FONT = new wxFont(12, wxSWISS, wxNORMAL, wxNORMAL);
  wxRED_PEN = new wxPen("RED", 3, wxSOLID);
  wxCYAN_PEN = new wxPen("CYAN", 3, wxSOLID);
  wxGREEN_PEN = new wxPen("GREEN", 1, wxSOLID);
  wxBLACK_PEN = new wxPen("BLACK", 1, wxSOLID);
  wxTRANSPARENT_PEN = new wxPen("BLACK", 1, wxTRANSPARENT);
  wxBLACK_DASHED_PEN = new wxPen("BLACK", 1, wxSHORT_DASH);

  wxBLUE_BRUSH = new wxBrush("BLUE", wxSOLID);
  wxGREEN_BRUSH = new wxBrush("GREEN", wxSOLID);
  wxWHITE_BRUSH = new wxBrush("WHITE", wxSOLID);
  wxBLACK_BRUSH = new wxBrush("BLACK", wxSOLID);
  wxTRANSPARENT_BRUSH = new wxBrush("BLACK", wxTRANSPARENT);
  wxCYAN_BRUSH = new wxBrush("CYAN", wxSOLID);
  wxRED_BRUSH = new wxBrush("RED", wxSOLID);
  wxGREY_BRUSH = new wxBrush("GREY", wxSOLID);
  wxMEDIUM_GREY_BRUSH = new wxBrush("MEDIUM GREY", wxSOLID);
  wxLIGHT_GREY_BRUSH = new wxBrush("LIGHT GREY", wxSOLID);

  wxBLACK = new wxColour("BLACK");
  wxWHITE = new wxColour("WHITE");
  wxRED = new wxColour("RED");
  wxBLUE = new wxColour("BLUE");
  wxGREEN = new wxColour("GREEN");
  wxCYAN = new wxColour("CYAN");
  wxLIGHT_GREY = new wxColour("LIGHT GREY");

  wxSTANDARD_CURSOR = new wxCursor(wxCURSOR_ARROW);
  wxHOURGLASS_CURSOR = new wxCursor(wxCURSOR_WAIT);
  wxCROSS_CURSOR = new wxCursor(wxCURSOR_CROSS);
}

// Pens

wxbPen::wxbPen(void)
{
  __type = wxTYPE_PEN;
}

wxbPen::~wxbPen()
{
}

wxbPen::wxbPen(wxColour& col, int Width, int Style)
{
  __type = wxTYPE_PEN;
}

wxbPen::wxbPen(char *col, int Width, int Style)
{
  __type = wxTYPE_PEN;
}

int wxbPen::GetWidth(void)
{
  return width;
}

int wxbPen::GetStyle(void)
{
  return style;
}

wxColour& wxbPen::GetColour(void)
{
  return *colour;
}

// Brushes

wxbBrush::wxbBrush(void)
{
  __type = wxTYPE_BRUSH;
}

wxbBrush::~wxbBrush()
{
}

wxbBrush::wxbBrush(wxColour& col, int Style)
{
  __type = wxTYPE_BRUSH;
}

wxbBrush::wxbBrush(char *col, int Style)
{
  __type = wxTYPE_BRUSH;
}

int wxbBrush::GetStyle(void)
{
  return style;
}

wxColour& wxbBrush::GetColour(void)
{
  return *colour;
}

// Icons
wxbIcon::wxbIcon(char bits[], int width, int height)
{
  __type = wxTYPE_ICON;
}

wxbIcon::wxbIcon(char *icon_file)
{
  __type = wxTYPE_ICON;
}

wxbIcon::~wxbIcon(void)
{
}

wxGDIList::wxGDIList(void)
{
}

wxGDIList::~wxGDIList(void)
{
  wxNode *node = First();
  while (node)
  {
    wxObject *object = (wxObject *)node->Data();
    wxNode *next = node->Next();
    delete object;
    node = next;
  }
}

// Cursors
// NB Should delete loaded Windows cursors with DestroyCursor!!!
wxbCursor::wxbCursor(char bits[], int width, int height)
{
  __type = wxTYPE_CURSOR;
}

wxbCursor::wxbCursor(char *cursor_file)
{
  __type = wxTYPE_CURSOR;
}

// Cursors by stock number
wxbCursor::wxbCursor(int cursor_type)
{
  __type = wxTYPE_CURSOR;
}

wxbCursor::~wxbCursor(void)
{
}

// Pen and Brush lists
wxPenList::~wxPenList(void)
{
  wxNode *node = First();
  while (node)
  {
    wxPen *pen = (wxPen *)node->Data();
    delete pen;
    node = First();
  }
}

void wxPenList::AddPen(wxPen *pen)
{
  Append(pen);
}

void wxPenList::RemovePen(wxPen *pen)
{
  DeleteObject(pen);
}

wxPen *wxPenList::FindOrCreatePen(wxColour *colour, int width, int style)
{
  wxPen *found = NULL;
  wxNode *node = First();
  while (node && !found)
  {
    wxPen *each_pen = (wxPen *)node->Data();
    if (each_pen->width == width && each_pen->style == style &&
        each_pen->colour && each_pen->colour->Red() == colour->Red() &&
        each_pen->colour->Green() == colour->Green() &&
        each_pen->colour->Blue() == colour->Blue())
      found = each_pen;
    node = node->Next();
  }
  if (found)
    return found;
  else
    return new wxPen(*colour, width, style);
}

wxPen *wxPenList::FindOrCreatePen(char *colour, int width, int style)
{
  wxColour *the_colour = wxTheColourDatabase->FindColour(colour);
  if (colour)
    return FindOrCreatePen(the_colour, width, style);
  else return NULL;
}

wxBrushList::~wxBrushList(void)
{
  wxNode *node = First();
  while (node)
  {
    wxBrush *brush = (wxBrush *)node->Data();
    delete brush;
    node = First();
  }
}

void wxBrushList::AddBrush(wxBrush *brush)
{
  Append(brush);
}

wxBrush *wxBrushList::FindOrCreateBrush(wxColour *colour, int style)
{
  wxBrush *found = NULL;
  wxNode *node = First();
  while (node && !found)
  {
    wxBrush *each_brush = (wxBrush *)node->Data();
    if (each_brush->style == style &&
        each_brush->colour && each_brush->colour->Red() == colour->Red() &&
        each_brush->colour->Green() == colour->Green() &&
        each_brush->colour->Blue() == colour->Blue())
      found = each_brush;
    node = node->Next();
  }
  if (found)
    return found;
  else
    return new wxBrush(*colour, style);
}

wxBrush *wxBrushList::FindOrCreateBrush(char *colour, int style)
{
  wxColour *the_colour = wxTheColourDatabase->FindColour(colour);
  if (colour)
    return FindOrCreateBrush(the_colour, style);
  else return NULL;
}


void wxBrushList::RemoveBrush(wxBrush *brush)
{
  DeleteObject(brush);
}

wxPoint::wxPoint(void)
{
}

wxPoint::wxPoint(float the_x, float the_y)
{
  x = the_x;
  y = the_y;
}

wxbBitmap::wxbBitmap(char bits[], int the_width, int the_height, int no_bits)
{
  __type = wxTYPE_BITMAP;
}

wxbBitmap::wxbBitmap(char *bitmap_file)
{
  __type = wxTYPE_BITMAP;
}

wxbBitmap::wxbBitmap(int w, int h, int d)
{
  __type = wxTYPE_BITMAP;
}

wxbBitmap::~wxbBitmap(void)
{
}
