/*
 * File:     wb_mdiag.cc
 * Purpose:  wxModalDialogBox
 *
 *                       wxWindows 1.40
 * Copyright (c) 1993 Artificial Intelligence Applications Institute,
 *                   The University of Edinburgh
 *
 *                     Author: Julian Smart
 *                       Date: 18-4-93
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice, author statement and this permission
 * notice appear in all copies of this software and related documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS,
 * IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL THE ARTIFICIAL INTELLIGENCE APPLICATIONS INSTITUTE OR THE
 * UNIVERSITY OF EDINBURGH BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF
 * DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * wxModalDialogBox behaves like wxDialogBox with modal set to TRUE
 * but takes care of dismissing the dialog on OK and CANCEL. Same approach
 * as wxMessageBox. Create the modal dialogbox with type wxOK, wxCANCEL etc.
 * Add any relevant dialogitems and call Handle(). Handle returns the result
 * code wich it either wxOK, wxCANCEL, etc.
 */

#include "wx_mdiag.h"
#include "wx_buttn.h"


wxModalDialogBox::wxModalDialogBox(wxFrame *Parent, char *Title, int buttons, int x, int y, int width, int height)
  : wxDialogBox(Parent, Title, TRUE,  x,  y,  width,  height)
{ dialog_buttons = buttons;
}

wxModalDialogBox::~wxModalDialogBox(void)
{
}

int wxModalDialogBox::Handle(void)
{
  // All items must have been placed before this call
  NewLine();

  wxButton *ok = NULL;
  wxButton *cancel = NULL;
  wxButton *yes = NULL;
  wxButton *no = NULL;

  if (dialog_buttons & wxYES_NO)
  {
    yes = new wxButton(this, (wxFunction)&wxDialogYesButton, "Yes");
    no = new wxButton(this, (wxFunction)&wxDialogNoButton, "No");
  }

  if (dialog_buttons & wxOK)
  {
    ok = new wxButton(this, (wxFunction)&wxDialogOkButton, "OK");
  }

  if (dialog_buttons & wxCANCEL)
  {
    cancel = new wxButton(this, (wxFunction)&wxDialogCancelButton, "Cancel");
  }

  Fit();

  if (ok && !cancel && !yes && !no)
    ok->Centre();

  Centre(wxBOTH);    // ?? Only if x, y are -1

  if (ok)
    ok->SetDefault();
  else if (yes)
    yes->SetDefault();

  wxDialogButtonPressed = 0;
  Show(TRUE);

  return wxDialogButtonPressed;
}
