#include "wx_resed.h"
#include "wx_sysev.h"
#include "wx_main.h"

void wxResGrpChange(wxListBox& groups, wxCommandEvent& event);
void wxResKeyChange(wxListBox& keys, wxCommandEvent& event);
void wxResValChange(wxText& value, wxCommandEvent& event);
void wxResSave(wxButton& save, wxEvent& event);
void wxResCancel(wxButton& cancel, wxEvent& event);

wxResEditDialog::wxResEditDialog(char *file, char *title, Bool modal, int x, int y, int width, int height)
  : wxDialogBox(NULL, title, modal, x, y, width, height)
{
  if(file)
	resfile = copystring(file);
  else resfile = NULL;
  curresource = NULL;
  grplist = NULL;
  keylist = NULL;
  valuefield = NULL;
}

wxResEditDialog::~wxResEditDialog(void)
{ wxNode *node;
  wxResource *res;

  if(resfile) delete resfile;
  node = reslist.First();
  while(node)
  { res = (wxResource *) node->Data();
    delete res;
    node = node->Next();
  }
}

void wxResEditDialog::AddResource(char *group, char *key)
{ wxList *newlist;
  wxResource *res;

  if(!grplist) return;

  newlist = wxTheApp->GetResourceList(group, key, resfile);

  if(newlist)
  { wxNode *node = newlist->First();
    while(node)
    { res = (wxResource *) node->Data();
      reslist.Append((wxObject *) res);
      if(grplist->FindString(res->group) < 0)
	grplist->Append(res->group);
      node = node->Next();
    }
    delete newlist;
  }
}

wxResEditDialog *wxResourceEditor(char *file, char *title, Bool modal, int x,
		   int y, int width, int height)
{ wxResEditDialog *resdialog = new wxResEditDialog(file, title, modal, 5, 5, 60, 20);

  if(!resdialog) return NULL;

  resdialog->SetLabelPosition(wxVERTICAL);
  resdialog->grplist = new wxListBox(resdialog, (wxFunction) wxResGrpChange, "Groups:", wxSINGLE, -1, -1, MAXGRPLEN, -1);
  resdialog->keylist = new wxListBox(resdialog, (wxFunction) wxResKeyChange, "Keys:", wxSINGLE, -1, -1, MAXKEYLEN, -1);
  resdialog->NewLine();
  resdialog->valuefield = new wxText(resdialog, (wxFunction) wxResValChange, "Value:", "", -1, -1, MAXVALLEN);
//  helpfield = new wxMultiText(resdialog, NULL, "Info:", -1, -1, 20, 3);
  resdialog->NewLine();
  wxButton *save = new wxButton(resdialog, (wxFunction) wxResSave, "Save");
  wxButton *cancel = new wxButton(resdialog, (wxFunction) wxResCancel, "Cancel");

  return resdialog;
}

void wxResGrpChange(wxListBox& groups, wxCommandEvent& event)
{ wxResEditDialog *resed;
  wxListBox *keylist;
  wxText *value;

  resed = (wxResEditDialog *) groups.GetParent();
  resed->curresource = NULL;
  keylist = resed->keylist;
  value = resed->valuefield;

  keylist->Show(FALSE);
  keylist->Clear();

  wxNode *node = resed->reslist.First();
  while(node)
  { wxResource *res = (wxResource *) node->Data();
    if(strcmp(res->group, event.commandString) == 0)
    { if(keylist->FindString(res->key) < 0)
	keylist->Append(res->key);
    }
    node = node->Next();
  }

  value->SetValue("");
  keylist->Show(TRUE);
}

void wxResKeyChange(wxListBox& keys, wxCommandEvent& event)
{ wxResEditDialog *resed;
  wxListBox *grplist;
  wxText *value;
  char *curgroup;

  resed = (wxResEditDialog *) keys.GetParent();
  value = resed->valuefield;
  grplist = resed->grplist;
  curgroup = grplist->GetStringSelection();

  if(!curgroup)
  { wxMessageBox("You must fist select a group.", "Error", wxOK);
    return;
  }

  value->SetValue("");

  wxNode *node = resed->reslist.First();
  while(node)
  { wxResource *res = (wxResource *) node->Data();
    if((strcmp(res->group, curgroup) == 0) &&
      (strcmp(res->key, event.commandString) == 0))
    { resed->curresource = res;
      break;
    }
    node = node->Next();
  }

  if(!resed->curresource) return;

  value->SetValue(resed->curresource->value);
  value->Show(TRUE);
}

void wxResValChange(wxText& value, wxCommandEvent& event)
{ wxResEditDialog *resed;
  wxResource *curresource;

  resed = (wxResEditDialog *) value.GetParent();
  curresource = resed->curresource;

  if(!curresource)
  { wxMessageBox("You must first select a group and key.", "Error", wxOK);
    return;
  }

  if(curresource->value) delete curresource->value;
  wxStripLTSpaces(event.commandString, wxBuffer);
  curresource->value = copystring(wxBuffer);
}

void wxResSave(wxButton& save, wxEvent& event)
{ wxResEditDialog *resed = (wxResEditDialog *) save.GetParent();
  wxNode *node;

  node = resed->reslist.First();
  while(node)
  { wxResource *res = (wxResource *) node->Data();
    wxTheApp->SetStringResource(res->group, res->key, res->value, resed->resfile);
    node = node->Next();
  }

  resed->Show(FALSE);
  delete resed;
}

void wxResCancel(wxButton& cancel, wxEvent& event)
{ wxResEditDialog *resed = (wxResEditDialog *) cancel.GetParent();

  resed->Show(FALSE);
  delete resed;
}

