#include "curswin.h"
#include "wx_utils.h"

#include <iostream.h>

ScrWnd *TheScreen = NULL;
wxWindow *FocusWnd = NULL;
wxWindow *FocusRootWnd = NULL;
CursWnd *FocusCurRootWnd = NULL;
CursWnd *FocusCurWnd = NULL;

char outbuffer[100]; // used by vsprintf

#ifdef CURSESX
#define MAXPUSHBACK 10

int pushbackbuf[MAXPUSHBACK];
int pushback = 0;

int pb_getch(void)
{
  if(pushback)
    return pushbackbuf[--pushback];
  else return getch();
}

void pb_ungetch(int c)
{
  if(pushback < MAXPUSHBACK - 1)
    pushbackbuf[pushback++] = c;
}
#endif

CursWnd::CursWnd(void)
{
  BorderStyle = 0;
  CursorStyle = CUR_HIDE;
  wx_window = NULL;
}

CursWnd::CursWnd(CursWnd *parent, wxRect& winRect, int btype, wxWindow *win)
{ 
  Parent = parent ? parent : TheScreen;  
  Parent->AddChild(this);
  Visible = FALSE;
  BorderStyle = btype;
  CursorStyle = CUR_HIDE;
  WinAttribs = A_NORMAL;
  ValidateWinRect(winRect);
  WinRect = winRect;
  wx_window = win;
  Handle = newwin(winRect.H, winRect.W, winRect.Y, winRect.X);

  InitWin();
}

void CursWnd::InitWin(void)
{
  if(BorderStyle)
  { Border(BorderStyle);
//    wmove(Handle, 1,1);
    wsetscrreg(Handle, 1, WinRect.H-2); // Don't scroll the top/bottom border
  }
  leaveok(Handle, FALSE);
  scrollok(Handle, FALSE);
  wattrset(Handle, WinAttribs);
}

CursWnd::~CursWnd()
{
  if(this != TheScreen) // endwin already called in ~ScrWnd
  { if (Parent)
    {	Parent->RemoveChild(this);
    }
    if(FocusCurWnd == this)
    { FocusCurWnd = NULL; FocusWnd = NULL;
    }
    if(FocusCurRootWnd == this)
    { FocusCurRootWnd = NULL; FocusRootWnd = NULL;
    }
    TheScreen->InvalidateRect(ScreenRect());
    if(Handle)
    { delwin(Handle);
      Handle = NULL;
    };
  }
}

// PRIVATE MEMBER FUNCTIONS

void CursWnd::AddChild(CursWnd *child)
{
  Children.Append(child);
}

void CursWnd::RemoveChild(CursWnd *child)
{
  Children.DeleteObject(child);
}

/*
void CursWnd::ValidateWinRect(wxRect& rect)
{ int minsize = BorderStyle ? 3 : 1;
  int pborder = Parent->BorderStyle ? 1 : 0; 	// parent border width/heigth
  wxRect prect = Parent->WinRect;
  
  rect.X = max(rect.X, prect.X + pborder);	// Window starts in parent
  rect.Y = max(rect.Y, prect.Y + pborder);
  rect.W = max(rect.W, minsize);		// Minimal size 1x1
  rect.H = max(rect.H, minsize);
  if ((rect.X + rect.W) > (prect.X + prect.W - pborder))	// Window doesn't extend parent
	rect.W = prect.X + prect.W - pborder - rect.X ;
  if ((rect.Y + rect.H) > (prect.Y + prect.H - pborder))
	rect.H = prect.Y + prect.H - pborder - rect.Y ;
  // We might have create a size smaller then the min size
  // Try moveing the window minsize back. Might be more than necessary!!
  if (rect.W < minsize)
	rect.X = max(prect.X + pborder, rect.X - minsize);
  if (rect.H < minsize)
	rect.Y = max(prect.Y + pborder, rect.Y - minsize);
}
*/

Bool CursWnd::NeedUpdate(CursWnd *wnd)
{
  return TheScreen->UpdateReg.Overlaps(wnd->WinRect);
}

wxRect CursWnd::ScreenRect(void)
{ wxRect srect(WinRect);
  wxNode *node;
  CursWnd *cwin;

  // if children are not contained in parent winrect
  node = Children.First();
  while(node)
  { cwin = (CursWnd *) node->Data();
    srect = Union(srect, cwin->WinRect);
    node = node->Next();
  }

  return srect;
}

void CursWnd::ValidateWinRect(wxRect& rect)
{ wxRect prect(TheScreen->WinRect); // or the Parent ?

  if (!rect.Inside(prect))
  { if (rect.X < 0)
		rect.X = prect.X; // Not left of screen
	if (rect.Y < 0)
		rect.Y = prect.Y; // Not top of screen
	if (rect.W > prect.W)
		rect.W = prect.W; // Not wider than screen
	if (rect.H > prect.H)
		rect.H = prect.H; // Not higher than screen
	if (rect.X + rect.W > prect.X + prect.W)
		rect.X = prect.X + prect.W - rect.W; // Not right of screen
	if (rect.Y + rect.H > prect.Y + prect.H)
		rect.Y = prect.Y + prect.H - rect.H; // Not bot of screen
  };
}

void CursWnd::RemoveChildren(void)
{ wxNode *node = Children.First();
  CursWnd *win;

  for (;node;node=node->Next())
  { win = (CursWnd *) node->Data();
    delete win;
  };
}

void CursWnd::Move(int dx, int dy)
{ wxNode *node = Children.First();
  CursWnd *win;

  WinRect.X += dx;
  WinRect.Y += dy;
  ValidateWinRect(WinRect);
  mvwin(Handle, WinRect.Y, WinRect.X);
 
// Move the children too
  for (; node; node = node->Next())
  { win = (CursWnd *) node->Data();
    if(win)
	win->Move(dx, dy);
  }
}

void CursWnd::Resize(int nw, int nh)
{ WINDOW *nwin;
  int lastr, lastc;

  if ((nwin = newwin(nh, nw, WinRect.Y, WinRect.X)) == NULL)
	return ;
  lastr = min(WinRect.H, nh) - 1; // Handle grow and shrink
  lastc = min(WinRect.W, nw) - 1;

#ifdef CURSESX
  overwrite(Handle, nwin);
#else
  if (BorderStyle) // Don't copy the borders.
	copywin(Handle, nwin, 1, 1, 1, 1, lastr - 1, lastc - 1, FALSE);
  else // copy the entire window
	copywin(Handle, nwin, 0, 0, 0, 0, lastr, lastc, FALSE);
#endif

  WinRect.W = nw;
  WinRect.H = nh;

  delwin(Handle);
  Handle = nwin;

  InitWin();
}

void CursWnd::Refresh(int ty, int by)
{ wxNode *node = Children.First();
  CursWnd *win;
  int top, nolines;
  if (!Visible)
	return;

// Translate Y and H to line start and no lines
  if (TheScreen->NeedUpdate(this))
  { 
#ifdef CURSESX
    touchwin(Handle);
#else
    top = max(ty, WinRect.Y) ;
    nolines = min(by, WinRect.Y + WinRect.H) - top;
    touchline(Handle, top - WinRect.Y, nolines);
#endif
    wnoutrefresh(Handle);
  };

  FocusCurWnd = this; // The last one is the focus wnd

  for (; node; node = node->Next())
  { win = (CursWnd *) node->Data();
    if(win)
	win->Refresh(ty, by);
  }
}

// PUBLIC MEMBER FUNCTIONS

void CursWnd::SetFocus(CursWnd *wnd)
{
  Children.DeleteObject(wnd);
  Children.Append(wnd);
  TheScreen->InvalidateRect(wnd->ScreenRect());
}

void CursWnd::BringToTop(void)
{
  TheScreen->SetFocus(this);
};

void CursWnd::SetCursor(int type)
{
  CursorStyle = type;
}

wxRect CursWnd::GetWinRect(void)
{
  return WinRect;
}

wxRect CursWnd::GetClientRect(void)
{ wxRect cr(WinRect);
  
  if(BorderStyle)
  { cr.X++;cr.Y++;cr.W-=2,cr.H-=2;
  }
  return cr;
}

void CursWnd::SetWinRect(wxRect& winRect)
{ 
  ValidateWinRect(winRect); 

  if (Visible)
  { TheScreen->InvalidateRect(ScreenRect());
    TheScreen->InvalidateRect(winRect);
  }
  
  if (WinRect.X != winRect.X || WinRect.Y != winRect.Y)
	   { wxRect moveRect(winRect - WinRect);
    Move(moveRect.X, moveRect.Y);
  }
  
  if (WinRect.W != winRect.W || WinRect.H != winRect.H)
  	Resize(winRect.W, winRect.H);
}

void CursWnd::GetPos(int *x, int *y)
{ *x = WinRect.X;
  *y = WinRect.Y;
}

void CursWnd::SetPos(int x, int y)
{ wxRect newRect(WinRect);
  
  newRect.X = x;
  newRect.Y = y;
  SetWinRect(newRect);
}

void CursWnd::GetSize(int *w, int *h)
{ *w = WinRect.W;
  *h = WinRect.H;
}

void CursWnd::GetClientSize(int *w, int *h)
{ *w = WinRect.W;
  *h = WinRect.H;

  if(BorderStyle)
  { *w -= 2;
    *h -= 2;
  };
}

void CursWnd::SetSize(int w, int h)
{ wxRect newRect(WinRect);
  
  newRect.W = w;
  newRect.H = h;
  SetWinRect(newRect);
}

void CursWnd::Clear(void)
{
  werase(Handle); // Clears the border too
  if(BorderStyle)
	Border(BorderStyle);
  TheScreen->InvalidateRect(WinRect);
}

void CursWnd::Scroll(int nolines)
{ int ox, oy;

  scrollok(Handle, TRUE);

  if (nolines > 0)	// Scrolling up
  { while (nolines--)
    	scroll(Handle);
  }
  else			// Scrolling down
  {
    getyx(Handle, oy, ox);	// curent cursor pos
    if(BorderStyle)
	wmove(Handle, 1,1);
    else wmove(Handle, 0,0);
    while (nolines++)
	winsertln(Handle); // Scroll down
    wmove(Handle, oy, ox); // restore cursor pos 
  }
  scrollok(Handle, FALSE);

  if(BorderStyle) // Restore the border lines
	Border(BorderStyle);

  TheScreen->InvalidateRect(WinRect);
}

void CursWnd::Border(int btype)
{ int oldattrib = WinAttrib();

  if(!BorderStyle) // If not created with border don't allow change
	return;

  BorderStyle = btype;

  if(BorderStyle & CW_BOLD_LINE)
  { WinAttribOn(A_BOLD);
    btype &= ~CW_BOLD_LINE;
  }

  switch(btype)
  { case CW_THICK_LINE:
#ifdef PDCURSES
			wborder(Handle, 0xBA,0xBA,0xCD,0xCD,0xC9,0xBB,0xC8,0xBC);
			box(Handle, ACS_VLINE, ACS_HLINE);
#else
			box(Handle, ACS_VLINE, ACS_HLINE);
#endif
			break;
    case CW_THIN_LINE:
			box(Handle, ACS_VLINE, ACS_HLINE);
			break;
  };

  WinAttrib(oldattrib);
  TheScreen->InvalidateRect(WinRect);
}

void CursWnd::Show(Bool show)
{
  TheScreen->InvalidateRect(ScreenRect());
  Visible = show;
}

int CursWnd::WinAttrib(void)
{
  return WinAttribs;
}

void CursWnd::WinAttrib(int winattrib)
{
  WinAttribs = winattrib;
  wattrset(Handle, WinAttribs);
}

void CursWnd::WinAttribOn(int winattrib)
{
  WinAttribs |= winattrib;
  wattrset(Handle, WinAttribs);
}

void CursWnd::WinAttribOff(int winattrib)
{
  WinAttribs &= ~winattrib;
  wattrset(Handle, WinAttribs);
}

void CursWnd::GotoXY(int x, int y)
{ int b = BorderStyle ? 1 : 0;

  if((x>=b) && (y>=b) && (x<=WinRect.W-b-1) && (y <=WinRect.H-b-1))
	wmove(Handle, y, x);
  wnoutrefresh(Handle);
}

void CursWnd::GetXY(int *x, int *y)
{
  getyx(Handle, *y, *x);
}

void CursWnd::CurLeft(void)
{ int x, y;
  
  GetXY(&x, &y);
  GotoXY(x-1, y);
}

void CursWnd::CurRight(void)
{ int x, y;
  
  GetXY(&x, &y);
  GotoXY(x+1, y);
}

void CursWnd::CurUp(void)
{ int x, y;
  
  GetXY(&x, &y);
  GotoXY(x, y-1);
}

void CursWnd::CurDown(void)
{ int x, y;
  
  GetXY(&x, &y);
  GotoXY(x, y+1);
}

void CursWnd::Print(char c)
{ 
  outbuffer[0] = c;
  outbuffer[1] = '\0';
  Print(outbuffer);
}

void CursWnd::Print(char *s)
{ int b = BorderStyle ? 1 : 0;
  int l, cx, cy;
  char *invalidc;

  getyx(Handle, cy, cx);
  l = min(strlen(s), WinRect.W-b-cx); // We don't want to print in the border!
  strncpy(outbuffer, s, l);
  outbuffer[l] = '\0';
  if(invalidc = strpbrk(outbuffer, "\n\r\t\v\f"))
	*invalidc = '\0';
  waddstr(Handle, outbuffer);
  TheScreen->InvalidateRect(wxRect(WinRect.X+cx,WinRect.Y+cy,l-1,1));
}

void CursWnd::Printf(char *fmt, ...)
{ va_list args;

  va_start(args, fmt);
  vsprintf(outbuffer, fmt, args);
  va_end(args);
  Print(outbuffer);
}

void CursWnd::Print(int x, int y, char c)
{
  GotoXY(x,y);
  Print(c);
}

void CursWnd::Print(int x, int y, char* s)
{
  GotoXY(x,y);
  Print(s);
}

void CursWnd::Printf(int x, int y, char *fmt, ...)
{ va_list args;

  GotoXY(x,y);
  va_start(args, fmt);
  vsprintf(outbuffer, fmt, args);
  va_end(args);
  Print(outbuffer);
}

void CursWnd::Print(int style, char c)
{ 
  outbuffer[0] = c;
  outbuffer[1] = '\0';
  Print(style, outbuffer);
}

void CursWnd::Print(int style, char *s)
{
  wattrset(Handle, style);
  Print(s);
  wattrset(Handle, WinAttribs); // restore the winattribs
}

void CursWnd::Printf(int style, char *fmt, ...)
{ va_list args;

  va_start(args, fmt);
  vsprintf(outbuffer, fmt, args);
  va_end(args);
  Print(style, outbuffer);  
}

void CursWnd::Print(int style, int x, int y, char c)
{
  GotoXY(x,y);
  Print(style, c);
}

void CursWnd::Print(int style, int x, int y, char *s)
{
  GotoXY(x, y);
  Print(style, s);
}

void CursWnd::Printf(int style, int x, int y, char *fmt, ...)
{ va_list args;

  GotoXY(x,y);
  va_start(args, fmt);
  vsprintf(outbuffer, fmt, args);
  va_end(args);
  Print(style, outbuffer);
}

// Non Clipping version that don't check any thing !
void CursWnd::PrintNC(char c)
{ 
  outbuffer[0] = c;
  outbuffer[1] = '\0';
  PrintNC(outbuffer);
}

void CursWnd::PrintNC(char* s)
{ int ox, oy;

  getyx(Handle, oy, ox);
  waddstr(Handle, s);
  TheScreen->InvalidateRect(wxRect(ox, oy, strlen(s), 1));
}

void CursWnd::PrintfNC(char *fmt, ...)
{ va_list args;

  va_start(args, fmt);
  vsprintf(outbuffer, fmt, args);
  va_end(args);
  PrintNC(outbuffer);
}

void CursWnd::PrintNC(int x, int y, char c)
{
  wmove(Handle, y, x);
  outbuffer[0] = c;
  outbuffer[1] = '\0';
  PrintNC(outbuffer);  
}

void CursWnd::PrintNC(int x, int y, char *s)
{
  wmove(Handle, y, x);
  PrintNC(s);  
}

void CursWnd::PrintfNC(int x, int y, char *fmt, ...)
{ va_list args;

  wmove(Handle, y, x);
  va_start(args, fmt);
  vsprintf(outbuffer, fmt, args);
  va_end(args);
  PrintNC(outbuffer);
}

void CursWnd::PrintNC(int style, char c)
{ 
  outbuffer[0] = c;
  outbuffer[1] = '\0';
  PrintNC(style, outbuffer);
}

void CursWnd::PrintNC(int style, char* s)
{ int ox, oy;

  getyx(Handle, oy, ox);
  wattrset(Handle, style);
  waddstr(Handle, s);
  wattrset(Handle, WinAttribs);
  TheScreen->InvalidateRect(wxRect(WinRect.X+ox, WinRect.Y+oy, strlen(s), 1));
}

void CursWnd::PrintfNC(int style, char *fmt, ...)
{ va_list args;

  va_start(args, fmt);
  vsprintf(outbuffer, fmt, args);
  va_end(args);
  PrintNC(style, outbuffer);
}

void CursWnd::PrintNC(int style, int x, int y, char c)
{
  wmove(Handle, y, x);
  outbuffer[0] = c;
  outbuffer[1] = '\0';
  PrintNC(style, outbuffer);  
}

void CursWnd::PrintNC(int style, int x, int y, char *s)
{
  wmove(Handle, y, x);
  PrintNC(style, s);  
}

void CursWnd::PrintfNC(int style, int x, int y, char *fmt, ...)
{ va_list args;

  wmove(Handle, y, x);
  va_start(args, fmt);
  vsprintf(outbuffer, fmt, args);
  va_end(args);
  PrintNC(style, outbuffer);
}

// ScrWnd Class

ScrWnd::ScrWnd(void)
{ 
  if (TheScreen)
	return;
  initscr();
  raw();
  noecho();
  nodelay(stdscr, TRUE);
  nonl();
  keypad(stdscr, TRUE);
#ifdef NCURSES
  wclear(stdscr);
  wrefresh(stdscr);
#endif
  Handle = stdscr;
  Parent = NULL;
  TheScreen = this;
  WinRect = wxRect(0, 0, COLS, LINES);
  UpdateReg = WinRect;
  DoUpdate = TRUE;
  StatusLine = NULL;
  scrollok(Handle, FALSE);
  leaveok(Handle, FALSE);
  SetCursor(CUR_HIDE);
}

ScrWnd::~ScrWnd()
{ 
  RemoveChildren();
  if(StatusLine)
	delwin(StatusLine);
  flushinp();
  noraw();
  nl();
  echo();
  touchwin(stdscr);
  wrefresh(stdscr);
  reset_shell_mode();
  endwin();
}

void ScrWnd::CreateStatusLine(void)
{
  if(StatusLine)
	return;

  WinRect.H--; // We have one line less
  StatusLine = newwin(1, COLS, LINES-1, 0);
  wattrset(StatusLine, A_REVERSE);
  mvwprintw(StatusLine, 0, 0, "%*s", COLS, " ");
}

void ScrWnd::StatusMsg(char *fmt, ...)
{ va_list args;

  if(!StatusLine) return;

  va_start(args, fmt);
  mvwprintw(StatusLine, 0, 0, "%*s", COLS, " ");  // Clear it
  mvwprintw(StatusLine, 0, 0, fmt, args);
  va_end(args);
}

void ScrWnd::StatusMsg(int style, char *fmt, ...)
{ va_list args;

  if(!StatusLine) return;
  wattron(StatusLine, style);
  va_start(args, fmt);
  werase(StatusLine);
  mvwprintw(StatusLine, 0, 0, fmt, args);
  va_end(args);
  wattroff(StatusLine, style);
}

Bool ScrWnd::KeyAvail(void)
{ int c;

#ifdef CURSESX
  c = pb_getch();
#else
  c = getch();
#endif
  if (c == ERR)
	return FALSE;
  else
  { 
#ifdef CURSESX
    pb_ungetch(c);
#else
    ungetch(c);
#endif
    return TRUE;
  }
}

int ScrWnd::WaitKey(void)
{ int key;

  nodelay(stdscr, FALSE); // be nice for other users
#ifdef CURSESX
  key = pb_getch();
#else
  key = getch();
#endif
  nodelay(stdscr, TRUE);

  return key;
}

int ScrWnd::PeekKey(void)
{ int c;

#ifdef CURSESX
  c = pb_getch();
#else
  c = getch();
#endif
  if (c != ERR)
#ifdef CURSESX
    pb_ungetch(c);
#else
    ungetch(c);
#endif
  return c;
}

int ScrWnd::GetKey(void)
{
#ifdef CURSESX
  return pb_getch();
#else
  return getch();
#endif
}

void ScrWnd::InvalidateRect(wxRect rect)
{
  if ((UpdateReg.W == 0) || (UpdateReg.H == 0))
	UpdateReg = rect;
  else
	UpdateReg = Union(UpdateReg, rect);
}

void ScrWnd::Refresh(int ty, int by)
{
#ifdef CURSESX
  touchwin(stdscr);
#else
  touchline(stdscr, ty, by - ty);
#endif
  wnoutrefresh(stdscr);
}

void ScrWnd::Clear(void)
{
  wclear(Handle);
  wrefresh(Handle);
  InvalidateRect(WinRect);
}

void ScrWnd::RefreshStatusLine(void)
{
  if(StatusLine)
  { // touchwin(StatusLine);
    wnoutrefresh(StatusLine);
  }
}

void ScrWnd::RefreshCursor(void)
{
#ifndef CURSESX
  if(!FocusCurWnd) return;

  if(FocusCurWnd->CursorStyle)
  { int cx, cy;

    curs_set(FocusCurWnd->CursorStyle);
    getyx(FocusCurWnd->Handle, cy, cx);
#ifdef CURSESX
    touchwin(FocsCurWnd->Handle);
#else
    touchline(FocusCurWnd->Handle, cy, 1);
#endif
    wnoutrefresh(FocusCurWnd->Handle);
  }
#endif
}

void ScrWnd::Refresh(void)
{ wxNode *node;
  CursWnd *win;
  int updTop, updBot;

  UpdateReg.X = 0; UpdateReg.W = COLS-1; // Right now we refresh entire lines
  updTop = UpdateReg.Y;
  updBot = UpdateReg.Y + UpdateReg.H;

  SetCursor(CUR_HIDE);
  Refresh(updTop, updBot);
  RefreshStatusLine();

  FocusCurRootWnd = NULL;
  FocusCurWnd = NULL;

  for (node=Children.First(); node; node = node->Next())
  { win = (CursWnd *) node->Data();
    if(win)
    { win->Refresh(updTop, updBot);
      FocusCurRootWnd = win;
    }
  };
  FocusWnd = FocusCurWnd ? FocusCurWnd->wx_window : NULL;
  FocusRootWnd = FocusCurRootWnd ? FocusCurRootWnd->wx_window : NULL;

  RefreshCursor();
  if (DoUpdate)
	doupdate();
  UpdateReg = wxRect(0,0,0,0);
}

void ScrWnd::Update(Bool yn)
{
  DoUpdate = yn;
  if (DoUpdate);
	doupdate();
}

void ScrWnd::SetCursor(int type)
{
#ifndef CURSESX
  curs_set(type);
#endif
}

