#include <termios.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <string.h>


typedef int Bool ;
#define TRUE 1
#define FALSE 0

struct termios termsettings;
int pfd, cfd, pid;
char *pttyname;
#define copystring(a) strdup(a)

Bool GiveUpControllingTTY(void)
{ int mypid;

  close(0);
  close(1);
  close(2);

  setsid();
/*
  mypid = getpid();
  tcsetpgrp(0, mypid);
*/

/*
  int fd;
  if((fd = open("/dev/tty", O_RDWR)) >= 0)
  {
    ioctl(fd, TIOCNOTTY, (char *) 0);
    close(fd);
  }
*/

  return TRUE;
}

Bool SetUpControllingTTY(void)
{
  int mypid = getpid();
#ifdef sgi
  setpgrp();
#else
  setpgrp(0, mypid);
#endif
  tcsetpgrp(0, mypid);

  return TRUE;
}

#ifdef sgi
Bool CreatePseudoTerminal(void)
{ char *line;
  void *oldsig;

  oldsig = signal(SIGCLD, SIG_DFL);
  line = _getpty(&pfd, O_RDWR, 0600, 0);
  signal(SIGCLD, oldsig);

  if(line)
  { pttyname = copystring(line);
    return TRUE;
  }
  else return FALSE;
/*
  if(line)
  { if((cfd = open(line, O_RDWR | O_NOCTTY)) != -1)
    { close(cfd);
      pttyname = copystring(line);
      return TRUE;
    }
    else
    { close(pfd);
      return FALSE;
    }
  }
  else return FALSE;
*/
}
#else /* sgi */

Bool CreatePseudoTerminal(void)
{ DIR *ptydir;
  struct dirent *entry;
  char ptyname[128];
  char ttyname[128];
  char entryname[128];

  ptydir = opendir("/dev");
  if(!ptydir) return FALSE;

  while(TRUE)
  { entry = readdir(ptydir);
    if(!entry) return FALSE;

    if(strncmp(entry->d_name, "pty", 3)) == 0)
    {
      wxPathMerge(ptyname, NULL, PTYDIR, entry->d_name, NULL);
      strcpy(entryname, entry->d_name);
      entryname[0] = 't';
      wxPathMerge(ttyname, NULL, PTYDIR, entryname, NULL);
      if((pfd = open(ptyname, O_RDWR)) != -1)
      { if((cfd = open(ttyname, O_RDWR | O_NOCTTY)) != -1)
        { pttyname = copystring(ttyname);
          close(cfd); 
          return TRUE;
        }
        else close(pfd);
      }
    }
  }

  return FALSE;
}
#endif

int main(void)
{
  int pip[2];
  char done[2];
  int real_uid, real_gid;
  int maxfiles;

  real_uid = getuid();
  real_gid = getgid();
  tcgetattr(0, &termsettings);

  if(!CreatePseudoTerminal()) return FALSE;

  fcntl(pfd, F_SETFL, FNDELAY);

  pipe(pip);
  pid = fork();

  if(pid < 0)
        return FALSE;

  if(pid == 0)
  {  int i;

     signal(SIGHUP, SIG_DFL);
     signal(SIGINT, SIG_DFL);
     signal(SIGQUIT, SIG_DFL);
     signal(SIGTERM, SIG_DFL);
#ifndef sgi
     signal(SIGTTIN, SIG_DFL);
     signal(SIGTTOU, SIG_DFL);
#endif

     setgid(real_gid);
     setuid(real_uid);


     if(!GiveUpControllingTTY())
     { write(pip[1], "no", 2);
       exit(-1);
     }

     cfd = open(pttyname, O_RDWR | O_NDELAY);
     if(cfd < 0)
     { write(pip[1], "no", 2);
       exit(errno);
     }

     dup2(cfd, 0);
     dup2(cfd, 1);
     dup2(cfd, 2);

     if(!SetUpControllingTTY())
     { write(pip[1], "no", 2);
       exit(-3);
     }

     tcsetattr(0, TCSADRAIN, &termsettings);

     write(pip[1], "go", 2);
     maxfiles = getdtablesize() ;
     for(i = 3; i < maxfiles; i++) close(i);

     printf("Child has access to tty\n");
     exit(0);
  }
  else /* the parent */
  {
     read(pip[0], done, 2);
     write(0, done , 2);
     printf("\n");
     while(read(pfd, done, 1) > 0)
	write(0, done, 1);
     close(cfd);
     close(pip[0]);
     close(pip[1]);
     waitpid(pid, 0, 0);
  }
}
