#include "winmgr.h"
#include "wx_main.h"
#include "wx_rect.h"
#include "wx_utils.h"
#include "curswin.h"
#include "wx_utils.h"
#include "wx_frame.h"
#include "wx_dialg.h"
#include "wx_list.h"

#include <ctype.h> // isdigit()

CurWinMgr *TheWinMgr = NULL;

CurWinMgr::CurWinMgr(void)
{
  Grabs = new wxList();
}

CurWinMgr::~CurWinMgr(void)
{ wxNode *node = wxTopLevelWindows.First();

  // Remove all toplevel windows not yet deleted
  for(;node;node=node->Next())
  { wxWindow *win = (wxWindow *) node->Data();
    delete win;
  }

  delete Grabs;
}

wxFrame* CurWinMgr::FindWindow(int id)
{ wxNode *node = wxTopLevelWindows.First();
  wxFrame *win;

  for(;node;node = node->Next())
  { win = (wxFrame *) node->Data();
    if(win->win_id == id)
	return win;
  };

  return NULL;
}

void CurWinMgr::HandleFocus(void)
{
  TheScreen->StatusMsg("Type the number of the new focus window. (ESC=Cancel)");
  TheScreen->Refresh();

  wxKeyEvent *wm_ev = wxTheApp->GetEvent();

  if (isdigit(wm_ev->keyCode))
  { int wi = wm_ev->keyCode - (int) '0';
	wxFrame *win = FindWindow(wi);
	if (win)
	{ win->Show(TRUE); // also sets the focus
	}
	else beep();
  }
  else beep();

  delete wm_ev;
  TheScreen->StatusMsg("");
  TheScreen->Refresh();
}

void CurWinMgr::HandleResize(void)
{ wxFrame *win = (wxFrame *) wxFocusFrame;
  CursWnd *sframe;
  Bool notdone = TRUE;

  if(!win) return;
  TheScreen->StatusMsg("Use arrow keys to resize the window. Finish with ESC.");
  TheScreen->Refresh();

  wxRect srect(win->GetWinRect());
  sframe = new CursWnd(NULL, srect, CW_THIN_LINE, NULL);
  if(!sframe) return;
  sframe->WinAttrib(A_BOLD);
  sframe->Show(TRUE);
  TheScreen->Refresh();

  while(notdone)
  { wxKeyEvent *r_ev = wxTheApp->GetEvent();
	switch(r_ev->keyCode)
	{ case KEY_UP:	srect.H -= 1;
			wxFocusFrame->ValidateWinRect(srect);
			sframe->SetWinRect(srect);
			TheScreen->Refresh();
			break;
	  case KEY_DOWN:	srect.H += 1;
			wxFocusFrame->ValidateWinRect(srect);
			sframe->SetWinRect(srect);
			TheScreen->Refresh();
			break;
	  case KEY_LEFT:	srect.W -= 1;
			wxFocusFrame->ValidateWinRect(srect);
			sframe->SetWinRect(srect);
			TheScreen->Refresh();
			break;
	  case KEY_RIGHT:	srect.W += 1;
			wxFocusFrame->ValidateWinRect(srect);
			sframe->SetWinRect(srect);
			TheScreen->Refresh();
			break;
	  case KEY_ESC:	notdone = FALSE;
			win->SetSize(srect.X, srect.Y, srect.W, srect.H);
			break;
	  default:	beep();
	};
	delete r_ev;
  };
  delete sframe;
  TheScreen->StatusMsg("");
  TheScreen->Refresh();
}

void CurWinMgr::HandleMove(void)
{
  if(!wxFocusFrame) return;

  CursWnd *cwin = wxFocusFrame->handle;
  wxRect mrect = cwin->GetWinRect();

  TheScreen->StatusMsg("Use arrow keys to move the window. Finish with ESC.");
  TheScreen->Refresh();

  while(1)
  { wxKeyEvent *mv_ev = wxTheApp->GetEvent();
	switch(mv_ev->keyCode)
	{ case KEY_UP:	mrect.Y -= 1;
			cwin->SetWinRect(mrect);
			TheScreen->Refresh();
			break;
	  case KEY_DOWN:	mrect.Y += 1;
			cwin->SetWinRect(mrect);
			TheScreen->Refresh();
			break;
	  case KEY_LEFT:	mrect.X -= 1;
			cwin->SetWinRect(mrect);
			TheScreen->Refresh();
			break;
	  case KEY_RIGHT:	mrect.X += 1;
			cwin->SetWinRect(mrect);
			TheScreen->Refresh();
			break;
	  case KEY_ESC:	delete mv_ev;
					return;
	  default:		beep();
	};
	delete mv_ev;
  };

  TheScreen->StatusMsg("");
  TheScreen->Refresh();
}

void CurWinMgr::HandleRefresh(void)
{
  TheScreen->Clear();
  TheScreen->Refresh();
}

void CurWinMgr::HandleClose(void)
{
  if (wxFocusFrame && (wxFocusFrame->OnClose()))
        delete wxFocusFrame;
}

void CurWinMgr::HandleQuit(void)
{
  wxExit();
}

void CurWinMgr::HandleHide(void)
{
  if(wxFocusFrame)
	wxFocusFrame->Iconize(TRUE);
}

void CurWinMgr::HandleUnHide(void)
{ wxStringList titlelist;
  wxNode *node;
  char **listitems;
  int nolistitems;

  node = wxIconizedWindows.First();
  while(node)
  { wxFrame *win = (wxFrame *) node->Data();
    titlelist.Add(win->GetTitle());
    node = node->Next();
  }

  nolistitems = titlelist.Number();
  if(nolistitems == 0)
  { wxMessageBox("There are no hidden windows.", "Win Mgr");
    return;
  }

  listitems = titlelist.ListToArray(FALSE);
  if(!listitems) return;

  int winid = wxGetSinglePopUpIndex("Hidden Windows:", "Win Mgr", nolistitems,
	    listitems, NULL, 0, 0, TRUE);
  node = wxIconizedWindows.Nth(winid);
  if(node)
  { wxFrame *newfwin = (wxFrame *) node->Data();
    newfwin->Iconize(FALSE);
  }

  delete listitems;
}

void CurWinMgr::HandleWMEvent(void)
{
  char *wmoptions[] = {"Focus", "Move", "Resize", "Hide", "Hidden Windows ...", "Refresh", "Quit!!"};
  int nowmoptions = 7;

  int req = wxGetSinglePopUpIndex("Options:", "Win Mgr", nowmoptions,
	    wmoptions, NULL, 0, 0, TRUE);

  switch(req)
  { case 0:	HandleFocus();
		break;
    case 1:	HandleMove();
		break;
    case 2:	HandleResize();
		break;
    case 3:	HandleHide();
		break;
    case 4:	HandleUnHide();
		break;
    case 5:	HandleRefresh();
		break;
    case 6:	HandleQuit();
		break;
    default:	break;
  };
}

Bool CurWinMgr::ProcessEvent(wxEvent *ev)
{ wxNode *grab = Grabs->Last();

  switch(ev->eventClass)
  {
	case wxTYPE_COMMAND_EVENT:
	     switch(ev->eventType)
	     {
	       case wxEVENT_TYPE_WM_COMMAND:
		    switch(((wxCommandEvent *)ev)->commandInt)
		    {
		      case WM_ACTIVATE:	if(!grab)
					{ HandleWMEvent();
					  return TRUE;
					}
					else return TRUE; // no wm events when focus grabbed
		    }
	     }
  }
  return FALSE;
}

void CurWinMgr::GrabFocus(wxWindow *win)
{
  Grabs->Append(win);
}

void CurWinMgr::ReleaseFocus(void)
{ wxWindow *win;
  wxNode *node;

  Grabs->DeleteNode(Grabs->Last()); // remove the last
/*
  node = Grabs->Last();             // set focus to previous grab
  if(node)
  { win = (wxWindow *) node->Data();
    win->SetFocus();
  }
  else win->FocusPreviousFrame();
*/
}

