/*
 * File:     wx_dc.cc
 * Purpose:  Device context implementation
 *
 *                       wxWindows 1.40
 * Copyright (c) 1993 Artificial Intelligence Applications Institute,
 *                   The University of Edinburgh
 *
 *                     Author: Julian Smart
 *                       Date: 18-4-93
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice, author statement and this permission
 * notice appear in all copies of this software and related documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS,
 * IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL THE ARTIFICIAL INTELLIGENCE APPLICATIONS INSTITUTE OR THE
 * UNIVERSITY OF EDINBURGH BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF
 * DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "common.h"
#include "wx_frame.h"
#include "wx_dc.h"
#include "wx_dccan.h"
#include "wx_dcmem.h"
#include "wx_dcps.h"
#include "wx_utils.h"
#include "wx_canvs.h"
#include "wx_dialg.h"
#include "wx_main.h"
#include "wx_rect.h"


// Default constructor
wxDC::wxDC(void)
{
}

wxDC::~wxDC(void)
{
}

void wxDC::SetClippingRegion(float cx, float cy, float cw, float ch)
{
      clipping = TRUE;
      user_cliprect.X = (int)cx;
      user_cliprect.Y = (int)cy;
      user_cliprect.W = (int)cw;
      user_cliprect.H = (int)ch;
}

void wxDC::DestroyClippingRegion(void)
{
  clipping = FALSE;
}

void wxDC::Clear(void)
{
  if(canvas)
	canvas->handle->Clear();
}

void wxDC::DrawLine(float x1, float y1, float x2, float y2)
{
}

void wxDC::DrawPoint(float x, float y)
{
}

void wxDC::DrawPolygon(int n, wxPoint points[], float xoffset, float yoffset)
{
}

void wxDC::DrawLines(int n, wxPoint points[], float xoffset, float yoffset)
{
}

void wxDC::DrawRectangle(float x, float y, float width, float height)
{
}

void wxDC::DrawRoundedRectangle(float x, float y, float width, float height, float radius)
{
}

void wxDC::DrawEllipse(float x, float y, float width, float height)
{
}

void wxDC::DrawIcon(wxIcon *icon, float x, float y)
{
}

void wxDC::SetFont(wxFont *the_font)
{
  font = the_font;
}

void wxDC::SetPen(wxPen *pen)
{
}

void wxDC::SetBrush(wxBrush *brush)
{
}

void wxDC::DrawText(char *text, float x, float y)
{
  if(!canvas) return;
  char *acopy = copystring(text);
  wxRect view = canvas->GetClientRect();
  view.X = canvas->xscroll_position;
  view.Y = canvas->yscroll_position;
  wxRect userrect = wxRect(x, y, strlen(acopy), 1);

  // first clip againts user clipping rect in scroll position
  if(clipping)
  { userrect = Intersect(userrect, user_cliprect);
    if(userrect.W == 0 || userrect.H == 0)
	return; // outside user clipping rect
  }

  // now clip against the view in scroll position
  userrect = Intersect(userrect, view);
  if(userrect.W == 0 || userrect.H == 0)
	return; // outside the view

  int dx = x - userrect.X;  // amount of clipping at front of string
  acopy[userrect.W] = '\0'; // clipping at end of string

  // translate scroll position to dev position
  userrect.X = userrect.X - canvas->xscroll_position;
  userrect.Y = userrect.Y - canvas->yscroll_position;

  // now clip against the clipping rect in dev position
  userrect = Intersect(userrect, cliprect);
  if(userrect.W == 0 || userrect.H == 0)
	return; // outside the update reg


  if(font)
	canvas->handle->Print(font->curs_attribs, userrect.X, userrect.Y, acopy+dx);
  else canvas->handle->Print(userrect.X, userrect.Y, acopy+dx);
}

void wxDC::SetBackground(wxBrush *brush)
{
}

void wxDC::SetLogicalFunction(int function)
{
}

Bool wxDC::StartDoc(char *message)
{
  return FALSE;
}

void wxDC::EndDoc(void)
{
}

void wxDC::StartPage(void)
{
}

void wxDC::EndPage(void)
{
}

void wxDC::SetMapMode(int mode)
{
}

void wxDC::SetUserScale(float x, float y)
{
}

float wxDC::DeviceToLogicalX(int x)
{
  return MS_XDEV2LOG(x);
}

float wxDC::DeviceToLogicalXRel(int x)
{
  return MS_XDEV2LOGREL(x);
}

float wxDC::DeviceToLogicalY(int y)
{
  return MS_YDEV2LOG(y);
}

float wxDC::DeviceToLogicalYRel(int y)
{
  return MS_YDEV2LOGREL(y);
}

int wxDC::LogicalToDeviceX(float x)
{
  return MS_XLOG2DEV(x);
}

int wxDC::LogicalToDeviceXRel(float x)
{
  return MS_XLOG2DEVREL(x);
}

int wxDC::LogicalToDeviceY(float y)
{
  return MS_YLOG2DEV(y);
}

int wxDC::LogicalToDeviceYRel(float y)
{
  return MS_YLOG2DEVREL(y);
}

Bool wxDC::Blit(float xdest, float ydest, float width, float height,
		wxCanvasDC *source, float xsrc, float ysrc, int rop)
{
  return FALSE;
}

float wxDC::GetCharHeight(void)
{
  return 1.0;
}

float wxDC::GetCharWidth(void)
{
  return 1.0;
}

void wxDC::GetTextExtent(char *string, float *x, float *y,
			     float *descent, float *externalLeading)
{
  *x = strlen(string);
  *y = 1;
  if(descent)
	*descent = 0;
  if(externalLeading)
	*externalLeading = 0;
}

wxCanvasDC::wxCanvasDC(void)
{
}

wxCanvasDC::wxCanvasDC(wxCanvas *the_canvas):wxbCanvasDC(the_canvas)
{
  canvas = the_canvas;
  if(canvas)
	cliprect = canvas->GetClientRect();
}

wxCanvasDC::~wxCanvasDC(void)
{
}

/*
 * Memory DC
 *
 */

wxMemoryDC::wxMemoryDC(void)
{
}

wxMemoryDC::wxMemoryDC(wxCanvasDC *old_dc):wxbMemoryDC(old_dc)
{
}

wxMemoryDC::~wxMemoryDC(void)
{
}

void wxMemoryDC::SelectObject(wxBitmap *bitmap)
{
}

