/*
 * File:     wx_main.cc
 * Purpose:  wxApp implementation
 *
 *                       wxWindows 1.40
 * Copyright (c) 1993 Artificial Intelligence Applications Institute,
 *                   The University of Edinburgh
 *
 *                     Author: Julian Smart
 *                       Date: 18-4-93
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice, author statement and this permission
 * notice appear in all copies of this software and related documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS,
 * IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL THE ARTIFICIAL INTELLIGENCE APPLICATIONS INSTITUTE OR THE
 * UNIVERSITY OF EDINBURGH BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF
 * DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#include "common.h"
#include "wx_frame.h"
#include "wx_menu.h"
#include "wx_main.h"
#include "wx_utils.h"
#include "wx_timer.h"
#include "winmgr.h"
#include "wx_dialg.h"
#include <termios.h>

#ifdef __BORLANDC__
#include <alloc.h>
#endif

Bool QuitMainLoop = FALSE; // used for modal dialogs
extern struct termios termsettings;

void wxInitialize(void)
{
  // save the terminal settings so it can be restored for terminal windows
  tcgetattr(0, &termsettings);
  TheScreen = new ScrWnd();
  TheWinMgr = new CurWinMgr();
  TheScreen->CreateStatusLine();
  wxCommonInit();
}

// Cleans up any wxWindows internal structures left lying around
void wxCleanUp(void)
{
  wxCommonCleanUp();
  RemoveAllTimers();
  delete TheWinMgr;
  delete TheScreen;
}

int main(int argc, char *argv[])
{
#ifdef __BORLANDC__
  unsigned long memfree;
  memfree = coreleft();
#endif

  wxInitialize();

  if (!wxTheApp)
  {
    printf("wxWindows error: You have to define an instance of wxApp!\n");
    return 0;
  }

  wxTheApp->argc = argc;
  wxTheApp->argv = argv;

  TheScreen->Update(FALSE);
  wxTheApp->wx_frame = wxTheApp->OnInit();

  if (wxTheApp->wx_frame)
  {  /* wxTheApp->wx_frame->Show(TRUE);
     if(wxTheApp->wx_frame->wx_menu_bar)
	wxTheApp->wx_frame->wx_menu_bar->SetFocus(); */
     TheScreen->Update(TRUE);
     TheScreen->Refresh();
     wxTheApp->MainLoop();
  }

  wxCleanUp();

#ifdef __BORLANDC__
  cout << "\n" << "Lost memory: " << (memfree - coreleft()) << "\n";
#endif
  return 0;
}

wxApp::wxApp(void)
{
  wx_frame = NULL;
  wxTheApp = this;
}

wxApp::~wxApp(void)
{
}

Bool wxApp::Initialized(void)
{
  if (wx_frame)
    return TRUE;
  else
    return FALSE;
}

Bool wxApp::EventAvail(void)
{
  return TheScreen->KeyAvail();
}

wxKeyEvent* wxApp::GetEvent(void)
{
  wxKeyEvent *the_event = new wxKeyEvent(wxEVENT_TYPE_CHAR);
  the_event->keyCode = TheScreen->WaitKey();
 
  return the_event;
}

wxEvent *wxApp::TranslateEvent(wxKeyEvent *ev)
{
  switch(ev->keyCode)
  { case KEY_F(1):
  	{ wxCommandEvent *wmev = new wxCommandEvent(wxEVENT_TYPE_WM_COMMAND);
	  wmev->commandInt = WM_ACTIVATE;
	  delete ev;
	  return wmev;
        }
    case KEY_F(2):
  	{ wxCommandEvent *wmev = new wxCommandEvent(wxEVENT_TYPE_WM_COMMAND);
	  wmev->commandInt = WM_HELP;
	  delete ev;
	  return wmev;
        }
    case CNTRL('N'):
        { wxCommandEvent *wmev = new wxCommandEvent(wxEVENT_TYPE_WM_COMMAND);
          wmev->commandInt = WM_FOCUSNEXT;
          delete ev;
          return wmev;
        }
    case CNTRL('B'):
        { wxCommandEvent *wmev = new wxCommandEvent(wxEVENT_TYPE_WM_COMMAND);
          wmev->commandInt = WM_FOCUSPREV;
          delete ev;
          return wmev;
        }
  }
  return ev;
}

int wxApp::MainLoop(void)
{ wxKeyEvent *kev;
  wxEvent *ev;
  while(!QuitMainLoop)
  {
	if(!wx_frame)
		break;

	while (OnIdle() && !EventAvail()) {}  // Wait for key event
	kev = GetEvent();
	ev = TranslateEvent(kev);  // key seq -> command
	if (!ProcessEvent(ev))
		break;
	delete ev;
  };

  QuitMainLoop = FALSE; // used for modal dialogs
  return 0;
}

Bool PreProcessEvent(wxEvent *event)
{ wxNode *node = NULL;
  wxWindow *pre_wnd;
  wxList *handlers;

  handlers = ((wxWindow *)event->eventObject)->preEventHandlers;
  if(handlers)
	node = handlers->First();

  while(node)
  { pre_wnd = (wxWindow *) node->Data();
    if(pre_wnd->PreProcessEvent(event))
	return TRUE;
    node = node->Next();
  };

  return FALSE;
}

Bool PostProcessEvent(wxEvent *event, Bool handled)
{ wxNode *node = NULL;
  wxWindow *post_wnd;
  wxList *handlers;

  handlers = ((wxWindow *)event->eventObject)->postEventHandlers;
  if(handlers)
	node = handlers->First();

  while(node)
  { post_wnd = (wxWindow *) node->Data();
    if(post_wnd->PostProcessEvent(event, handled))
	return TRUE;
    node = node->Next();
  };

  return FALSE;
}

Bool wxApp::ProcessEvent(wxEvent *event)
{ Bool handled;
  wxWindow *wnd;

  if(!wx_frame)
	return FALSE;  // No main frame so quit

// Is it a window manager event?
  if (TheWinMgr->ProcessEvent(event))
	return TRUE;

  if (!wx_frame)
	return FALSE;

// Work backwards from the focus wnd until handled
  for (wnd = wxFocusWnd;wnd;wnd = wnd->GetParent())
  { event->eventObject = wnd;
    if(PreProcessEvent(event))
	return TRUE;
    handled = wnd->ProcessEvent(event);
    handled |= PostProcessEvent(event, handled);
    if(handled)
	return TRUE;
  };

  return TRUE; // event not handled and ignored
}

Bool wxApp::OnIdle(void)
{
  return FALSE;
}

void wxExit(void)
{
 if (wxTheApp && wxTheApp->wx_frame)
   delete wxTheApp->wx_frame;
}

// Yield to incoming messages
Bool wxYield(void)
{
  sleep(1);
  return TRUE;
}

