#include "wx_privt.h"
#include "wx_frame.h"
#include "wx_buttn.h"
#include "wx_menu.h"
#include "winmgr.h"

wxRect wxMenuPanel::MinimalWinSize(void)
{ wxRect minRect;

  minRect.W = max_width;
  minRect.H = max_height;
  return minRect;
}

wxMenuPanel::wxMenuPanel(wxFrame *frame, int x, int y, int width, int height, int style, Bool create) :
   wxPanel(frame, x, y, width, height, style, FALSE)
{
  if(create)
	Create(frame, x, y, width, height, style);
}

Bool wxMenuPanel::Create(wxFrame *frame, int x, int y, int width, int height, int style)
{
  if (!frame) return FALSE;

  wxRect wrect(x, y, width ,height);
  ClientToScreen(wrect);

  if(style & wxBORDER)
  { wrect.W += 2; wrect.H += 2;
    handle = new CursWnd(frame->handle, wrect, CW_THIN_LINE, this);
  }
  else handle = new CursWnd(frame->handle, wrect, CW_NO_LINE, this);

  frame->AddChild(this);

  if(handle)
	return TRUE;
  else return FALSE;
}

void wxMenuPanel::OnSetFocus(void)
{
  if(defaultItem)
	defaultItem->SetFocus();
}

void wxMenuPanel::OnKillFocus(void)
{
  if(defaultItem)
	defaultItem->OnKillFocus();
}

void wxMenuPanel::Show(Bool flag)
{ wxFrame *parent = (wxFrame *) GetParent();

  if(flag)
  {// TheWinMgr->GrabFocus(this);
    wxPanel::Show(TRUE);
  }
  else
  { if(parent->wx_menu_bar->menu_bar_menu->menu_panel != this)
	wxPanel::Show(FALSE);
    // TheWinMgr->ReleaseFocus();
  }
}

// Class TextString

TextString::TextString(char *s)
{
  size = 0;
  len = 0;
  text = NULL;
  Set(s);
}

void TextString::Set(char *s)
{
  if(text)
	delete text;

  if(s)
	len = strlen(s);
  else len = 0;

  size = len+GROW_SIZE;
  text = new char[size+1];
  if(s && text)
  { text[0] = '\0';  // in case s == ""
    strcpy(text, s);
  }
}

TextString::~TextString()
{
  if(text)
	delete text;
}

int TextString::Length(void)
{
  return len;
}

char *TextString::SubString(int from, int to)
{
  if((from < 0) || (to >= len) || !text)
	return "";

  char *subs = new char[to-from+2];
  if(!subs)
	return "";

  memcpy(subs, text+from, to-from+1);
  subs[to-from+1] = '\0';
  return subs;
}

char *TextString::FromPos(int pos)
{
  if(!text || pos >= len)
	return "";
  else return text+pos;
}

void TextString::Append(char *s)
{
  Put(len, s);
}

void TextString::Put(int pos, char *s)
{
  if(s)
  { while(*s != '\0')
    { Put(pos, *s);
      s++;
      pos++;
    }
  }
}

void TextString::Put(int pos, char c)
{
  if((pos < 0) || (pos > len) || !text)
	return;

  if(pos < len) 		// normal put
	text[pos] = c;
  else				// append
  { if(!Grow())
	return;
    text[pos] = c;
    len++;
    text[len] = '\0';
  }
}

void TextString::Insert(int pos, char c)
{
  if((pos < 0) || (pos > len) || !text)
	return;

  if(pos == len)
	Put(pos, c); // append
  else
  { if(!Grow())
	return;
    memmove(text+pos+1,text+pos,len-pos); // Shift right from pos
    text[pos] = c;
    len++;
    text[len] = '\0';
  }
}

void TextString::Insert(int pos, char *s)
{
  if(s)
  { while(*s != '\0')
    { Insert(pos, *s);
      s++;
      pos++;
    }
  }
}

void TextString::Delete(int pos, int count)
{
  if(!text)
	return;

  if(len-pos>=count)
  { memmove(text+pos, text+pos+count, len-pos-count);
    len-=count;
    text[len] = '\0';
    Shrink();
  }
}

Bool TextString::Grow(void)
{ char *new_text = NULL;

  if(!text)
	return FALSE;

  if(len+1 <= size)
	return TRUE;

  new_text = new char[size+GROW_SIZE];
  if(!new_text)
	return FALSE;

  strcpy(new_text, text);
  delete(text);
  text = new_text;
  size += GROW_SIZE;

  return TRUE;
}

void TextString::Shrink(void)
{ char *new_text = NULL;

  if(!text)
	return;

  if(size - len > 2 * GROW_SIZE)
  { new_text = new char[size-GROW_SIZE];
    if(new_text)
    { strcpy(new_text, text);
      delete text;
      text = new_text;
      size -= GROW_SIZE;
    }
  }
}

TextString::operator char * ()
{
    if(text)
	return text;
    else return "";
}
