#include "wx_proc.h"
#include "wx_list.h"
#include "wx_utils.h"

#include <signal.h>
#include <sys/wait.h>
#include <dirent.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <errno.h>

/* Nowhere defined */
extern "C" 
{
int select(int width, fd_set *readfds, fd_set *write_fds, fd_set *exceptfds,
           struct timeval *timeout);
};

#define PTYDIR "/dev"
#define PTYNAME "pty"
#define MAXPTYLEN = 15

static Bool SigHandlerInstalled = FALSE;
wxList ProcessList;
int sig;
struct termios termsettings;

Bool GiveUpControllingTTY(void)
{

  close(0);
  close(1);
  close(2);

  setsid();

/*
  int fd;
  if((fd = open("/dev/tty", O_RDWR)) >= 0)
  {
    ioctl(fd, TIOCNOTTY, (char *) 0);
    close(fd);
  }
*/

  return TRUE;
}

Bool SetUpControllingTTY(void)
{ 
  int mypid = getpid();
#ifdef sgi
  setpgrp();
#else
  setpgrp(0, mypid);
#endif
  tcsetpgrp(0, mypid);

  return TRUE;
}

void wxChildSigHandler(int sig, int code, struct sigcontext *scp, char *addr)
{ wxNode *node;
  int procstatus;

  node = ProcessList.First();

  if(!node)
  { SigHandlerInstalled = FALSE;
    return;
  }

  while(node)
  { wxProcess *p = (wxProcess *) node->Data();
    if(waitpid(p->pid, &procstatus, WNOHANG | WUNTRACED) > 0)
    { p->pstatus = procstatus;
      p->Status();
    }
    node = node->Next();
  }

  signal(SIGCHLD, wxChildSigHandler);
}

wxProcess::wxProcess(char *appname, char *args[], char *env[], Bool needtty)
{
  pid = -1;
  status = wxPNOTSTARTED;
  statusmsg = "No status info";
  pstatus = -1;
  pneedtty = needtty;
  exit_status = -1;
  // bzero(&pinfo, sizeof(pinfo));
  pfd = -1;
  cfd = -1;
  pttyname = NULL;

  pargs = args;
  penv = env;
}

wxProcess::~wxProcess(void)
{
  Stop();
  if(pttyname) delete pttyname;
  ProcessList.DeleteObject(this);
  close(pfd);
}

char *wxProcess::ProcessName(void)
{ static char *invalid = "Invalid processname";

  if(pargs && pargs[0])
	return pargs[0];
  else return invalid;
}

#ifdef sgi
Bool wxProcess::CreatePseudoTerminal(void)
{ char *line;
  void *oldsig;

  oldsig = signal(SIGCLD, SIG_DFL);
  line = _getpty(&pfd, O_RDWR, 0600, 0);
  signal(SIGCLD, oldsig);

  if(line)
  { pttyname = copystring(line);
    return TRUE; // assume it is ok
  }
  else return FALSE;
/*
  if(line)
  { if((cfd = open(line, O_RDWR | O_NOCTTY)) != -1)
    { close(cfd);
      pttyname = copystring(line);
      return TRUE;
    }
    else
    { close(pfd);
      return FALSE;
    }
  }
  else return FALSE;
*/
}

#else /* sgi */

Bool wxProcess::CreatePseudoTerminal(void)
{ DIR *ptydir;
  struct dirent *entry;
  char ptyname[MAXFILELEN];
  char ttyname[MAXFILELEN];
  char entryname[MAXFILELEN];

  ptydir = opendir(PTYDIR);
  if(!ptydir) return FALSE;

  while(TRUE)
  { entry = readdir(ptydir);
    if(!entry) return FALSE;

    if(strncmp(entry->d_name, PTYNAME, strlen(PTYNAME)) == 0)
    {
      wxPathMerge(ptyname, NULL, PTYDIR, entry->d_name, NULL);
      strcpy(entryname, entry->d_name);
      entryname[0] = 't';
      wxPathMerge(ttyname, NULL, PTYDIR, entryname, NULL);
      if((pfd = open(ptyname, O_RDWR)) != -1)
      { if((cfd = open(ttyname, O_RDWR | O_NOCTTY)) != -1)
	{ pttyname = copystring(ttyname);
	  close(cfd); // don't let it become the controlling terminal
          return TRUE;
        }
        else close(pfd);
      }
    }
  }

  return FALSE;
}
#endif

Bool wxProcess::Execute(void)
{ int pip[2];
  char done[2];
  int real_uid, real_gid;

  if(status != wxPNOTSTARTED) return FALSE;

  if(!pargs) return FALSE;

  real_uid = getuid();
  real_gid = getgid();

  if(pneedtty)
  {  // Restore it after the child has executed
    savetty();
    if(!CreatePseudoTerminal()) return FALSE;

    // make the master pty non-blocking
    fcntl(pfd, F_SETFL, FNDELAY);
  }

  pipe(pip);
  pid = fork();

  if(pid < 0)
	return FALSE;

  if(pid == 0) // the child
  {  int i;
 
     signal(SIGHUP, SIG_DFL);
     signal(SIGINT, SIG_DFL);
     signal(SIGQUIT, SIG_DFL);
     signal(SIGTERM, SIG_DFL);
#ifndef sgi
     signal(SIGTTIN, SIG_DFL);
     signal(SIGTTOU, SIG_DFL);
#endif

     setgid(real_gid);
     setuid(real_uid);

     if(pneedtty)
     { 
       if(!GiveUpControllingTTY())
       { write(pip[1], "no", 2);
	 exit(-1);
       }

       cfd = open(pttyname, O_RDWR);
       if(cfd < 0)
       { write(pip[1], "no", 2);
	 exit(errno);
       }

       dup2(cfd, 0);
       dup2(cfd, 1);
       dup2(cfd, 2);
     }

     if(pneedtty)
     {
       if(!SetUpControllingTTY())
       { write(pip[1], "no", 2);
	 exit(-3);
       }
     }

// Needed on ultrix because default the settings are wrong!!
     tcsetattr(0, TCSADRAIN, &termsettings); 

     write(pip[1], "go", 2);
     int maxfiles = getdtablesize() ;
     for(i = 3; i < maxfiles; i++) close(i);

     execve(pargs[0], pargs, penv);
     exit(-4);	// if execve failed
  }
  else // the parent
  {  
     read(pip[0], done, 2); // wait for execve
     if(pneedtty)
     { close(cfd);
       // restore saved before the fork
       resetty();
     }
     close(pip[0]);
     close(pip[1]);
     ProcessList.Append(this);
     UpdateStatus();
     if(!SigHandlerInstalled)
	 signal(SIGCHLD, wxChildSigHandler);
  }
  return TRUE;
}

Bool wxProcess::ExecuteAndWait(void)
{ int err;
  int procstatus;

  if(Execute())
  { err = waitpid(pid, &procstatus, WUNTRACED);
    if((err == -1) && ((errno == ECHILD) || (errno == EFAULT)))
       status = wxPSTOPPED;
    else
    if(err > 0)
    { pstatus = procstatus;
      Status();
    }
    return TRUE;
  }
  else return FALSE;
}

void wxProcess::Suspend(void)
{
  if((pid > 0) && (status == wxPRUNNING))
      kill(pid, SIGSTOP);
}

void wxProcess::Continue(void)
{
  if((pid > 0) && (status == wxPSUSPENDED))
      kill(pid, SIGCONT);
}

void wxProcess::Stop(void)
{ int procstat;

  if((pid > 0) && (status != wxPSTOPPED))
  { kill(pid, SIGKILL); // ? SIGINT, SIGKILL, SIGQUIT
    // wait for proc to exit
    waitpid(pid, &procstat, WUNTRACED) ;
  }
}

void wxProcess::SetTerminalSize(int width, int height)
{
  struct winsize ws;
  if(ioctl(pfd, TIOCGWINSZ, &ws) >= 0)
  { ws.ws_row = height;
    ws.ws_col = width;
    ioctl(pfd, TIOCSWINSZ, &ws);
  
    if(status != wxPNOTSTARTED)
  	kill(pid, SIGWINCH);
  }
}

int wxProcess::UpdateStatus(void)
{ int err;
  int procstatus;
 
  if(status != wxPSTOPPED)
  { err = waitpid(pid, &procstatus, WNOHANG | WUNTRACED);
    if(err == 0)
    { status = wxPRUNNING;
      statusmsg = "Running";
    }
    else
    if((err == -1) && ((errno == ECHILD) || (errno == EFAULT)))
        status = wxPSTOPPED;
    else 
    { pstatus = procstatus;
      Status();
    }
  }
  return status;
}

int wxProcess::ExitStatus(void)
{
  return exit_status;
}

char *wxProcess::StatusMsg(void)
{
  Status();
  return statusmsg;
}

int wxProcess::Status(void)
{ 
  if(pstatus == -1) // no status info yet
	return status;

  if(WIFSIGNALED(pstatus))
  { sig = WTERMSIG(pstatus);
    switch(sig)
    { 
      case SIGKILL:	status = wxPSTOPPED;
			statusmsg = "Stopped (killed)";
			break;
      case SIGQUIT:	status = wxPSTOPPED;
			statusmsg = "Stopped (quit)";
			break;
      case SIGFPE:	status = wxPSTOPPED;
			statusmsg = "Stopped (arithmetic exception)";
			break;
      case SIGSEGV:	status = wxPSTOPPED;
			statusmsg = "Stopped (segmentation violation)";
			break;
      case SIGBUS:	status = wxPSTOPPED;
			statusmsg = "Stopped (bus error)";
			break;
      case SIGABRT:	status = wxPSTOPPED;
			statusmsg = "Stopped (abort)";
			break;
      case SIGILL:	status = wxPSTOPPED;
			statusmsg = "Stopped (illegal instruction)";
			break;
      case SIGTERM: 	status = wxPSTOPPED;
			statusmsg = "Stopped (terminated)";
                    	break;
      case SIGSTOP: 	status = wxPSUSPENDED;
			statusmsg = "Suspended";
	            	break;
      case SIGCONT: 	status = wxPRUNNING;
			statusmsg = "Running (continued)";
                    	break;
      default:		status = wxPSTOPPED;
			statusmsg = "Stopped (unknown)";
			break;
    }
  }
  else 
  if(WIFSTOPPED(pstatus))
  { sig = WSTOPSIG(pstatus);
    switch(sig)
    { 
      case SIGKILL:     status = wxPSTOPPED;
                        statusmsg = "Stopped (killed)";
                        break;
      case SIGQUIT:     status = wxPSTOPPED;
                        statusmsg = "Stopped (quit)";
                        break;
      case SIGFPE:      status = wxPSTOPPED;
                        statusmsg = "Stopped (arithmetic exception)";
                        break;
      case SIGSEGV:     status = wxPSTOPPED;
                        statusmsg = "Stopped (segmentation violation)";
                        break;
      case SIGBUS:      status = wxPSTOPPED;
                        statusmsg = "Stopped (bus error)";
                        break;
      case SIGABRT:     status = wxPSTOPPED;
                        statusmsg = "Stopped (abort)";
                        break;
      case SIGILL:      status = wxPSTOPPED;
                        statusmsg = "Stopped (illegal instruction)";
                        break;
      case SIGTERM:     status = wxPSTOPPED;
                        statusmsg = "Stopped (terminated)";
                        break;
      case SIGSTOP: 	status = wxPSUSPENDED;
			statusmsg = "Suspended";
	            	break;
      default:		status = wxPSTOPPED;
			statusmsg = "Stopped (unknown)";
			break;
    }
  }
  else
  if(WIFEXITED(pstatus))
  { status = wxPSTOPPED;
    statusmsg = "Exited";
    exit_status = WEXITSTATUS(pstatus);
  }

  return status;
}

void wxProcess::GetInfo(timeval *utime, timeval *stime, int *memusage)
{
/*
  if(utime)
      *utime = pinfo.ru_utime;
  if(stime)
      *stime = pinfo.ru_stime;
  if(memusage)
      *memusage = getpagesize() * pinfo.ru_maxrss;
*/
}

Bool wxProcess::InputPending(void)
{ fd_set readfds;
  struct timeval timeout;
  int iores;

   if(pfd < 0)
	return FALSE;

   FD_ZERO(&readfds);
   FD_SET(pfd, &readfds);
   bzero(&timeout, sizeof(struct timeval));
   timeout.tv_usec = 500;
   iores =  select(pfd+1, &readfds, 0, 0, &timeout);
   if(iores > 0)
   	return FD_ISSET(pfd, &readfds);
   else return FALSE;
}

int wxProcess::Write(char *s, int len)
{ int iores;

  iores = write(pfd, s, len);
  return iores;
}

int wxProcess::Write(char *s)
{ int iores;

  iores = write(pfd, s, strlen(s));
  return iores;
}

extern char outbuffer[];

int wxProcess::Writef(char *fmt, ...)
{ va_list args;
  int iores;

  va_start(args, fmt);
  vsprintf(outbuffer, fmt, args);
  va_end(args);
  iores = write(pfd, outbuffer, strlen(outbuffer));
  return iores;
}

int wxProcess::Write(char c)
{ int iores;

  iores = write(pfd, &c, 1);
  return iores;
}

int wxProcess::Read(char *s, int len)
{ int iores;

  iores = read(pfd, s, len);
  return iores;
}

int wxProcess::Read(char *c)
{ int iores;

  iores = read(pfd, c, 1);
  return iores;
}
