#include "wx_rect.h"
#include "wx_utils.h"

wxRect::wxRect(int x, int y, int w, int h)
{
  X = x; Y = y; W = w; H = h;
}

wxRect::wxRect(const wxRect& rect)
{
  X = rect.X;
  Y = rect.Y;
  W = rect.W;
  H = rect.H;
}

wxRect::wxRect(void)
{
  X = Y = W = H = 0;
}

Bool wxRect::Inside(int x, int y)
{
  return	(X <= x) &&
		(X + W >= x) &&
		(Y <= y) &&
		(Y + H >= y);
}

Bool wxRect::Inside(wxRect& rect)
{
  return	(X >= rect.X) &&
			(X + W <= rect.X + rect.W) &&
			(Y >= rect.Y) &&
			(Y + H <= rect.Y + rect.H);
}

Bool wxRect::Overlaps(wxRect& rect)
{ wxRect irect = Intersect(*this, rect);

  return irect.W != 0 && irect.H != 0;
}

wxRect wxRect::operator - (wxRect& rect)
{ wxRect newRect(*this);

  newRect.X -= rect.X;
  newRect.Y -= rect.Y;
  newRect.W -= rect.W;
  newRect.H -= rect.H;

  return newRect;
}

wxRect Intersect(wxRect& r1, wxRect& r2)
{ wxRect newRect;

  newRect.X = max(r1.X, r2.X);
  newRect.Y = max(r1.Y, r2.Y);
  newRect.W = min(r1.X+r1.W, r2.X+r2.W) - newRect.X;
  newRect.H = min(r1.Y+r1.H, r2.Y+r2.H) - newRect.Y;

  // if the don't overlap, width or height is negative
  if(newRect.W < 0 || newRect.H < 0)
	newRect.X = newRect.Y = newRect.W = newRect.H = 0;

  return newRect;
}

wxRect Union(wxRect& r1, wxRect& r2)
{ wxRect newRect;

  if(r1.W == 0 || r1.H == 0)
	return r2;
  if(r2.W == 0 || r2.H == 0)
	return r1;

  newRect.X = min(r1.X, r2.X);
  newRect.Y = min(r1.Y, r2.Y);
  newRect.W = max(r1.X + r1.W, r2.X + r2.W) - newRect.X;
  newRect.H = max(r1.Y + r1.H, r2.Y + r2.H) - newRect.Y;

  return newRect;
}

int operator == (wxRect& r1, wxRect& r2)
{
  return (r1.X == r2.X) &&
	 (r1.Y == r2.Y) &&
	 (r1.W == r2.W) &&
	 (r1.H == r2.H);
}
ostream& operator<<(ostream& os, wxRect& r)
{
  return os << "wxRect[" << r.X << ","  << r.Y << "," << r.W
            << "," << r.H << "]";
}
