#include "wx_termb.h"

wxTermBuf::wxTermBuf(int len)
{
  if(len == 0)
        len = DEFBUFLEN;

  buf = new char[len];
  blen = len;
  bput = 0;
  bget = 0;
  btry = -1;
  bfull = FALSE;
}

wxTermBuf::~wxTermBuf(void)
{
  if(buf) delete buf;
}

/*
   empty buffer:        put == get, full == FALSE
   full buffer:         put == get, full == TRUE
*/

Bool wxTermBuf::Put(char c)
{
  if((bput == bget) && bfull) // full buffer
        return FALSE;

  buf[bput++] = c;
  bput %= blen;

  if(bput == bget)
        bfull = TRUE;

  return TRUE;
}

Bool wxTermBuf::Get(char *c)
{ int *getp;

  if(Trying())
        getp = &btry;
  else getp = &bget;

  if((bput == *getp) && !bfull)         // empty buffer
        return FALSE;

  *c = buf[(*getp)++];
  (*getp) %= blen;

  if(*getp == bput)
        bfull = FALSE;

  return TRUE;
}

Bool wxTermBuf::Peek(char *c)
{ int *getp;

  if(Trying())
        getp = &btry;
  else getp = &bget;

  if((bput == *getp) && !bfull)         // empty buffer
        return FALSE;

  *c = buf[*getp];

  return TRUE;
}

int wxTermBuf::Put(char *s, int len)
{ int i;

  for(i = 0; i < len; i++)
  { if(!Put(s[i]))
        return i;
  }

  return len;
}

int wxTermBuf::Put(char *s)
{
  return Put(s, strlen(s));
}

void wxTermBuf::Try(void)
{
  btry = bget;
  bwasfull = bfull;
}

void wxTermBuf::Commit(void)
{
  bget = btry;
  btry = -1;
}

void wxTermBuf::Cancel(void)
{
  btry = -1;
  bfull = bwasfull;
}

void wxTermBuf::Reset(void)
{
  bput = bget = 0;
  btry = -1;
  bfull = FALSE;
}

