#include "wx_termw.h"
#include "wx_termb.h"
#include "wx_temu.h"
#include "wx_utils.h"
#include "curswin.h"
#include "common.h"

#include <fcntl.h>

wxTermWindow::wxTermWindow(wxFrame *parent, wxTermEmu *emu, 
   int x, int y, int width, int height, int style, Bool create) : wxWindow()
{ 
  inbuf = NULL;
  outbuf = NULL;
  termemu = NULL;
  logfile = NULL;
  if(create)
	Create(parent, emu, x, y, width, height, style);
}

wxTermWindow::~wxTermWindow(void)
{
  if(logfile) delete logfile;
  if(inbuf) delete inbuf;
  if(outbuf) delete outbuf;
  if(termemu) delete termemu;
}

Bool wxTermWindow::Create(wxFrame *frame, wxTermEmu *emu, 
   int x, int y, int width, int height, int style)
{ int cw, ch;

  if(!frame) return FALSE;
  window_parent = frame;
  termemu = emu;

  if(x == -1)
        x = 0;
  if(y == -1)
        y = 0;
  frame->GetClientSize(&cw, &ch);
  if(width == -1)
        width = cw;
  if(height == -1)
        height = ch;
  wxRect srect(x, y, width, height);
  ClientToScreen(srect);

  handle = new CursWnd(frame->handle, srect, CW_NO_LINE, this);
  if(!handle) return FALSE;
  handle->Show(TRUE);

  inbuf = new wxTermBuf();
  outbuf = new wxTermBuf();
  if(!inbuf || !outbuf) return FALSE;

  GetClientSize(&cw, &ch);

  emu->Initialize(inbuf, this, outbuf);

  frame->AddChild(this);
  return TRUE;
}

Bool wxTermWindow::Logging(void)
{
  return logfile != NULL;
}

void wxTermWindow::LogToFile(char *file)
{
  if(logfile)
  { delete logfile;
    logfile = NULL;
    close(log_fd);
  }
  if(file)
  { logfile = copystring(file);
	// force a create or truncation
    log_fd = open(logfile, O_RDWR | O_CREAT | O_TRUNC, 0600);	
  }
}

void wxTermWindow::Log(char *buf, int len)
{
  if(log_fd > -1)
	write(log_fd, buf, len);
}

void wxTermWindow::SetEmulation(wxTermEmu *emu)
{
  // flush the buffer first
  termemu->ParseProcessOutput(inbuf, this, outbuf);
  termemu = emu;
  emu->Initialize(inbuf, this, outbuf);
}

void wxTermWindow::OnSize(int w, int h)
{
  termemu->ResetSize();
}

void wxTermWindow::Reset(void)
{
  termemu->Reset(inbuf, this, outbuf);
}

void wxTermWindow::Put(char *buf, int len)
{ int i;

  if(Logging())
	Log(buf, len);

  for(i=0; i<len; i++)
  { if(!inbuf->Put(buf[i])) // the buf is full so flush it first
    { termemu->ParseProcessOutput(inbuf, this, outbuf);
      inbuf->Put(buf[i]);  // try again; if it fails the char is lost
    }
  }
  termemu->ParseProcessOutput(inbuf, this, outbuf);
}

int wxTermWindow::Get(char *buf, int len)
{ int act = 0;
  char c;

  while(outbuf->Get(&c) && (act < len))
  	buf[act++] = c;

  return act;
}

Bool wxTermWindow::ProcessEvent(wxEvent *ev)
{ wxKeyEvent *k_ev = (wxKeyEvent *) ev;

  if (k_ev->eventType == wxEVENT_TYPE_CHAR)
  { termemu->ParseProcessInput(k_ev->keyCode, outbuf);
    return TRUE;
  }
  else return wxWindow::ProcessEvent(ev);
}

