/*
 * File:     wx_text.cc
 * Purpose:  wxTextWindow implementation
 *
 *                       wxWindows 1.40
 * Copyright (c) 1993 Artificial Intelligence Applications Institute,
 *                   The University of Edinburgh
 *
 *                     Author: Julian Smart
 *                       Date: 18-4-93
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice, author statement and this permission
 * notice appear in all copies of this software and related documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS,
 * IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL THE ARTIFICIAL INTELLIGENCE APPLICATIONS INSTITUTE OR THE
 * UNIVERSITY OF EDINBURGH BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF
 * DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "common.h"
#include "wx_main.h"
#include "wx_text.h"
#include "wx_utils.h"
#include "wx_frame.h"
#include "wx_panel.h"
#include "wx_mtxt.h"

#include <iostream.h>
#include <fstream.h>
#include <stdio.h>


wxTextWindow::wxTextWindow(wxFrame *frame, int x, int y, int width, int height,
			   int style, Bool create):
  wxbTextWindow(frame, x, y, width, height, style)
{
  file_name = NULL;
  handle = NULL;

  if(create)
	Create(frame, x, y, width, height, style);
}

Bool wxTextWindow::Create(wxFrame *frame, int x, int y, int width, int height,
			   int style)
{
  if(!frame) return FALSE;

  frame->AddDisposal(this); // just to get deleted

  text_panel = new wxPanel(frame, x, y, width, height, style);
  if(!text_panel) return FALSE;

  window_parent = frame;
  handle = text_panel->handle;   // handle must be something !?

  int w, h;
  text_panel->GetClientSize(&w, &h);
  mtext_item = new wxMultiText(text_panel, NULL, NULL, NULL, 0, 0, w, h);
  mtext_item->Show(TRUE);
  if(!mtext_item) return FALSE;

  text_panel->Show(TRUE);
  return TRUE;
}

wxTextWindow::~wxTextWindow(void)
{
  if (file_name) delete file_name;
  handle = NULL; // to avoid deletion in ~wxbWin().
}

void wxTextWindow::SetSize(int x, int y, int w, int h)
{
  text_panel->SetSize(x,y,w,h);
  text_panel->GetClientSize(&w, &h);
  mtext_item->SetSize(0,0,w,h);
}

void wxTextWindow::GetSize(int *width, int *height)
{
  text_panel->GetSize(width, height);
}

void wxTextWindow::GetPosition(int *x, int *y)
{
  text_panel->GetPosition(x,y);
}

void wxTextWindow::OnSetFocus(void)
{
  mtext_item->OnSetFocus();
}

void wxTextWindow::OnKillFocus(void)
{
  mtext_item->OnKillFocus();
}

Bool wxTextWindow::LoadFile(char *file)
{
  if (file_name)
    delete file_name;

 file_name = copystring(file);
 Clear();

 FILE *in_file = fopen(file, "r");

 if (file)
 {
   while(fgets(wxBuffer, WXBUFLEN - 1, in_file) != NULL)
   {
     mtext_item->Append(wxBuffer);
   } 
   mtext_item->Refresh();
   return TRUE;
 }
 else return FALSE;
}

// If file is null, try saved file name first
// Returns TRUE if succeeds.
Bool wxTextWindow::SaveFile(char *file)
{
  return 0;
}

void wxTextWindow::WriteText(char *text)
{
  mtext_item->Append(text);
  mtext_item->Refresh();
}

void wxTextWindow::Clear(void)
{
}

Bool wxTextWindow::Modified(void)
{
  return FALSE;
}

// Not clear whether Clear is required as well as DiscardEdits
void wxTextWindow::DiscardEdits(void)
{
  Clear();
}

char *wxTextWindow::GetContents(void)
{
  return NULL;
}
