/*
 * File:     wx_timer.cc
 * Purpose:  wxTimer implementation (X version)
 *
 *                       wxWindows 1.40
 * Copyright (c) 1993 Artificial Intelligence Applications Institute,
 *                   The University of Edinburgh
 *
 *                     Author: Julian Smart
 *                       Date: 18-4-93
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice, author statement and this permission
 * notice appear in all copies of this software and related documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS,
 * IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL THE ARTIFICIAL INTELLIGENCE APPLICATIONS INSTITUTE OR THE
 * UNIVERSITY OF EDINBURGH BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF
 * DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <time.h>
#ifdef __unix__
#include <sys/time.h>
#endif
#include <signal.h>

#include "wx_timer.h"
#include "wx_list.h"
#include "wx_main.h"
#include "wx_frame.h"

wxList Timers;

int wxTimer_interval = 1;     // timer granularity

#if defined(__unix__) && !defined(__GO32__)
void wxTimerProc(void);
#endif

void RemoveAllTimers(void)
{ wxNode *node;
  wxTimer *timer;

  node = Timers.First();
  while(node)
  { timer = (wxTimer *) node->Data();
    delete timer;
    node = Timers.First();
  }
}

wxTimer::wxTimer(void)
{
  interval = 0;
  tick_count = 0;
}

wxTimer::~wxTimer(void)
{
  Stop();
}

Bool wxTimer::Start(int milliseconds)
{
#if defined(__unix__) && !defined(__GO32__)
  Timers.DeleteObject(this);
  Timers.Append(this);
  if(milliseconds == -1)
	milliseconds = 1000;

  interval = milliseconds;
  tick_count = milliseconds / wxTimer_interval;

  if(Timers.Number() == 1)
  { struct itimerval itime;
    itime.it_interval.tv_sec = 0;
    itime.it_interval.tv_usec = wxTimer_interval;
    itime.it_value.tv_sec = 0;
    itime.it_value.tv_usec = wxTimer_interval;
    setitimer(ITIMER_VIRTUAL, &itime, NULL);
    signal(SIGVTALRM, wxTimerProc);
  }
#endif
  return TRUE;
}

void wxTimer::Stop(void)
{
#if defined(__unix__) && !defined(__GO32__)
  Timers.DeleteObject(this);
  if(Timers.Number() == 0)
  { struct itimerval itime;
    itime.it_interval.tv_sec = 0;
    itime.it_interval.tv_usec = 0;
    itime.it_value.tv_sec = 0;
    itime.it_value.tv_usec = 0;
    setitimer(ITIMER_VIRTUAL, &itime, NULL); // disable timer
    signal(ITIMER_VIRTUAL, SIG_DFL); // reset default handler
  }
#endif
}

#if defined(__unix__) && !defined(__GO32__)
void wxTimerProc(void)
{ wxNode *node;
  wxTimer *timer;

  node = Timers.First();
  if(!node) return;

  while(node)
  {  timer = (wxTimer *) node->Data();
     if(--timer->tick_count <= 0)
     { timer->Notify();
       timer->tick_count = timer->interval;
     }
     node = node->Next();
  }
  signal(SIGVTALRM, wxTimerProc); // reset the timerhandler
}
#endif
