#include "wx_user.h"
#include "wx_utils.h"

#include <pwd.h>

extern "C" {
 int geteuid(void);
 int getuid(void);
 int pwdauth(char *user, char *passwd);
 int setuid(int uid);
 int setgid(int gid);
 int initgroups(char *user, int gid);
}

wxUser::wxUser(char *uname)
{
  name = home = shell = passwd = NULL;
  uid = gid = -1;

  struct passwd *pw;
  if(!uname)
	pw = getpwuid(getuid());	// current user info
  else pw = getpwnam(uname);
  if(!pw) return; 

  name = copystring(pw->pw_name);
  home = copystring(pw->pw_dir);
  uid = pw->pw_uid;
  gid = pw->pw_gid;
  passwd = copystring(pw->pw_passwd);
  shell = copystring(pw->pw_shell);
}


wxUser::~wxUser(void)
{
  if(name) delete name;
  if(home) delete home;
  if(passwd) delete passwd;
  if(shell) delete shell;
}

Bool wxUser::CheckPasswd(char *upasswd)
{ 
  if(geteuid() != 0)	// only root can check passwd
	return FALSE;

  return (strcmp(passwd, crypt(upasswd, passwd)) == 0);
}

Bool wxUser::Become(void)
{ 
  int userid = geteuid();
  if((userid != 0) || (userid != uid))
	return FALSE;

  setgid(gid);
  initgroups(name, gid);
  if(userid == uid)
	return TRUE;
  else return (setuid(uid) == 0);
}

char *wxUser::Name(void)
{
  return name;
}

char *wxUser::HomeDir(void)
{
  return home;
}

char *wxUser::Password(void)
{
  return passwd;
}

char *wxUser::Shell(void)
{
  return shell;
}

int wxUser::UserID(void)
{
  return uid;
}

int wxUser::GroupID(void)
{
  return gid;
}

