/*
 * File:     wx_utils.cc
 * Purpose:  Various utilities
 *
 *                       wxWindows 1.40
 * Copyright (c) 1993 Artificial Intelligence Applications Institute,
 *                   The University of Edinburgh
 *
 *                     Author: Julian Smart
 *                       Date: 18-4-93
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice, author statement and this permission
 * notice appear in all copies of this software and related documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, EXPRESS,
 * IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
 * MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * IN NO EVENT SHALL THE ARTIFICIAL INTELLIGENCE APPLICATIONS INSTITUTE OR THE
 * UNIVERSITY OF EDINBURGH BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT ADVISED OF THE POSSIBILITY OF
 * DAMAGE, AND ON ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH
 * THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdlib.h>
#include <fstream.h>
#include <dirent.h>
#ifdef __BORLANDC__
#include <alloc.h>
#endif

#include "common.h"
#include "wx_utils.h"

// Execute a command (e.g. another program) in a
// system-independent manner.
// Well - not quite. Note that under UNIX, the call will only
// return immediately if an ampersand is appended to the command.
// Under Windows, the call returns immediately.
Bool wxExecute(char *command)
{
#ifdef PDCURSES
  int error = system(command);
  TheScreen->InvalidateRect(TheScreen->GetWinRect());
  TheScreen->Refresh();
#else
  endwin();
  int error = system(command);
  doupdate();
#endif
  return error == 0;
}

void wxPathMerge(char *path, char *drive, char *dir, char *name, char *ext)
{
  if(!path) return;
  path[0] = '\0';
#ifdef __BORLANDC__
  fnmerge(path, drive, dir, name, ext);
#else
  int len;

  if(drive)
  { len = strlen(drive);
    strcat(path, drive);
    if((len > 0) && (drive[len-1] != ':'))
	strcat(path, ":");
  }
  if(dir)
  { len = strlen(dir);
    strcat(path, dir);
    if((len > 0) && ((dir[len-1] != '/') || (dir[len-1] != '\\')))
	strcat(path, DIRSEPSTR);
  }
  if(name)
  { strcat(path, name);
    if(ext)
    { len = strlen(ext);
      if(len > 0)
      { char *period = rindex(path, '.');
        if(period && (period != name))
	  *period = '\0'; // remove the period from the name
        if(ext[0] != '.')
		strcat(path, ".");
        strcat(path, ext);
      }
    }
  }
  // we dont allow an extension if there was no filename
  // turn the slashes in the right direction
  while(*path != '\0')
  { if((*path == '\\') || (*path == '/'))
	*path = DIRSEPCHAR;
    path++;
  }

#endif
}

void wxPathSplit(char *path, char *drive, char *dir, char *name, char *ext)
{
  if(!path) return;
#ifdef __BORLANDC__
  fnsplit(path, drive, dir, name, ext);
#else
  if(drive)
	drive[0] = '\0';
  if(dir)
  { char *tmp = PathOnly(path);
    if(tmp)
    { strcpy(dir, tmp);
      delete tmp;
    }
    else dir[0] = '\0';
  }
  if(name)
  { char *tmp = FileNameFromPath(path);
    if(tmp)
    { strcpy(name, tmp);
      delete tmp;
    }
    else name[0] = '\0';
  }
  if(ext)
  { if(name)
    { char *point = rindex(name, '.');
      if(point && (point != name)) // files starting with .
      { strcpy(ext, point+1);  // return ext without the point
	*point = '\0';  // remove ext from filename
      }
    }
    else ext[0] = '\0';
  }
#endif
}

// Get free memory in bytes, or -1 if cannot determine amount (e.g. on UNIX)
long wxGetFreeMemory(void)
{
#ifdef __BORLANDC__
  return coreleft();
#else
  return -1;
#endif
}

// Sleep for nSecs seconds under UNIX, do nothing under Windows
// XView implementation according to the Heller manual
void wxSleep(int nSecs)
{
}

// Consume all events until no more left
void wxFlushEvents(void)
{
  while(TheScreen->GetKey() != ERR) ;
}

Bool IsGroup(char *line, char *group)
{ char *eog;

  if(line[0] == '[')
  { *group = '\0';
    eog = rindex(line, ']');
    if(eog)
    { *eog = '\0';
      strncpy(group, line+1, MAXGRPLEN);
      group[MAXGRPLEN-1] = '\0';
      return TRUE;
    }
  }

  return FALSE;
}

Bool IsKeyValuePair(char *line, char *key, char *value)
{ char *equalsign;

  key[0] = value[0] = '\0';

  equalsign = index(line, '=');
  if(equalsign)
  { *equalsign = '\0';
    strncpy(key, line, MAXKEYLEN);
    key[MAXKEYLEN-1] = '\0';
    strncpy(value, ++equalsign, MAXVALLEN);
    value[MAXVALLEN-1] = '\0';
    char *nl = index(value, '\n');
    if(nl)
	*nl = '\0';
    return TRUE;
  }

  return FALSE;
}

// return's a list of all the groups and keys in the inifile
wxList *wxGetResourceList(char *inifile)
{ 
  if(!inifile) return new wxList(wxKEY_STRING);

  FILE *resfile = fopen(inifile, "r");
  if(!resfile) return NULL;

  wxList *resourcelist = new wxList(wxKEY_STRING);
  if(!resourcelist)
  { fclose(resfile);
    return NULL;
  }
  wxList *thekeylist = NULL;

  char thegroup[MAXGRPLEN];
  char thekey[MAXKEYLEN];
  char thevalue[MAXVALLEN];

  thegroup[0] = '\0';

  while(fgets(wxBuffer, WXBUFLEN, resfile))
  { if(IsGroup(wxBuffer, thegroup))
    { wxNode *groupnode = resourcelist->Find(thegroup);
      if(!groupnode)
      { thekeylist = new wxList(wxKEY_STRING);
	if(!thekeylist)
        { fclose(resfile);
          return resourcelist;
        }
	resourcelist->Append(thegroup, thekeylist);
      }
      else thekeylist = (wxList *) groupnode->Data();
    }
    else
    if(IsKeyValuePair(wxBuffer, thekey, thevalue) && thekeylist)
	thekeylist->Append(thekey, (wxObject *) copystring(thevalue));
  }

  fclose(resfile);
  return resourcelist;
}

Bool wxWriteResourceList(wxList *reslist, char *inifile)
{
  if(!inifile) return FALSE;

  FILE *resfile = fopen(inifile, "w");
  if(!resfile) return FALSE;

  wxNode *groupnode = reslist->First();
  while(groupnode)
  { wxList *keylist = (wxList *) groupnode->Data();
    wxNode *keynode = keylist->First();
    if(keynode)
    { fprintf(resfile, "[%s]\n", groupnode->key.string);
      while(keynode)
      { char *value = (char *) keynode->Data();
	fprintf(resfile, "%s=%s\n", keynode->key.string, value);
	keynode = keynode->Next();
      }
    }
    groupnode = groupnode->Next();
  }

  return fclose(resfile) == 0;
}

// return's the first group and key that match

char *wxGetResource(char *group, char *key, wxList *reslist)
{ 
  if(!reslist)	return NULL;

  wxNode *groupnode = reslist->Find(group);
  if(!groupnode) return NULL;

  wxList *keylist = (wxList *) groupnode->Data();
  wxNode *keynode = keylist->Find(key);

  if(!keynode) return NULL;
  else return (char *) keynode->Data();
}

/* puts or replaces the resource in  the list */
void wxSetResource(char *group, char *key, char *value, wxList *reslist)
{ 
  if(!reslist) return;

  wxNode *groupnode = reslist->Find(group);
  wxList *keylist;

  if(!groupnode)
  { keylist = new wxList(wxKEY_STRING);
    if(!keylist) return;
    keylist->DeleteContents(TRUE);
    reslist->Append(group, keylist);
  }
  else keylist = (wxList *) groupnode->Data();
  
  wxNode *keynode = keylist->Find(key);
  if(!keynode)
	keylist->Append(key, (wxObject *) value);
  else
  { char *oldval = (char *) keynode->Data();
    delete oldval;
    keynode->SetData((wxObject *) value);
  }
}

void wxDeleteResource(char *group, char *key, wxList *reslist)
{
  if(!reslist) return;

  wxNode *groupnode = reslist->Find(group);
  wxList *keylist;

  if(!groupnode) return;
  keylist = (wxList *) groupnode->Data();
  
  wxNode *keynode = keylist->First();
  while(keynode)
  { if(strcmp(keynode->key.string, key) == 0)
    { char *value = (char *) keynode->Data();
      delete value;
      delete keynode;
      return;
    }
    keynode = keynode->Next();
  }
}

char *wxGetStringResource(char *group, char *key, char *def, wxList *reslist)
{
  char *value = wxGetResource(group, key, reslist);
  if(value)
	return copystring(value);
  else
  if(def)
	 return copystring(def);
  else return NULL;
}

void wxSetStringResource(char *group, char *key, char *value, wxList *reslist)
{
  wxSetResource(group, key, copystring(value), reslist);
}

Bool wxGetBoolResource(char *group, char *key, Bool def, wxList *reslist)
{ char *strvalue;

   strvalue = wxGetResource(group, key, reslist);
   if(strvalue)
   	return strcmp(strvalue, "TRUE") == 0;
   else return def;
}

void wxSetBoolResource(char *group, char *key, Bool value, wxList *reslist)
{ char *strvalue;

  if(value)
	strvalue = "TRUE";
  else strvalue = "FALSE";

  wxSetResource(group, key, copystring(strvalue), reslist);
}

long wxGetIntResource(char *group, char *key, long def, wxList *reslist)
{ char *strvalue;
  long value = def;

   strvalue = wxGetResource(group, key, reslist);
   if(strvalue)
  	StringToLong(strvalue, &value);
 
  return value;
}

void wxSetIntResource(char *group, char *key, long value, wxList *reslist)
{ char *strvalue;

  strvalue = LongToString(value);
  wxSetResource(group, key, strvalue, reslist);
}

double wxGetFloatResource(char *group, char *key, double def, wxList *reslist)
{ char *strvalue;
  double value = def;

   strvalue = wxGetResource(group, key, reslist);
   if(strvalue)
	StringToDouble(strvalue, &value);
  
   return value;
}

void wxSetFloatResource(char *group, char *key, double value, wxList *reslist)
{ char *strvalue;

  strvalue = DoubleToString(value);
  wxSetResource(group, key, strvalue, reslist);
}
