#include "wx_vt100.h"
#include "wx_termb.h"
#include "wx_termw.h"
#include "wx_utils.h"
#include "curswin.h"

#include <ctype.h>

#define ResetArgs { bzero(args, sizeof(args)); argc = 0;}

wxVT100Emu::wxVT100Emu(void) : wxTermEmu()
{
  state = S_PRINT;
  G0 = A_NORMAL;
  G1 = A_ALTCHARSET;
  CharSet = G0;
  ResetArgs;
  top = bot = -1;
  wrap = TRUE;
}

void wxVT100Emu::Initialize(wxTermBuf *in, wxTermWindow *win, wxTermBuf *out)
{ int cw, ch;
  CursWnd *cwin = win->handle;

  win->GetClientSize(&cw, &ch);
  cwin->SetCursor(CUR_NORM);
  cwin->GotoXY(0, ch - 1);
}

void wxVT100Emu::WrapMode(Bool yn)
{
  wrap = yn;
}

void wxVT100Emu::Reset(wxTermBuf *in, wxTermWindow *win, wxTermBuf *out)
{
  in->Reset();
  out->Reset();
  win->handle->Clear();
  win->handle->WinAttrib(0);
  state = S_PRINT;
  top = bot = -1;
  win->Refresh();
}

void wxVT100Emu::ResetSize(void)
{
  top = bot = -1;
}

void wxVT100Emu::ParseProcessOutput(wxTermBuf *in, wxTermWindow *win, wxTermBuf *out)
{ int curx, cury, maxx, maxy;
  char c;
  int i; /* used for some loops */

  CursWnd *handle = win->handle;
  win->GetClientSize(&maxx, &maxy);
  maxx--; maxy--;
  if(top == -1)
  { top = 0; bot = maxy;
  }

  while(in->Get(&c))
  { handle->GetXY(&curx, &cury);
    switch(state)
    { case S_PRINT:
        switch(c)
        {
	  case '\0':	/* null*/
			break;
          case '\r':    /* Carrige return */
                        curx = 0;
                        handle->GotoXY(curx, cury);
                        break;
          case '\n':    /* Newline */
                        if(cury == bot)
                                handle->Scroll(1);
                        else cury++;
                        handle->GotoXY(curx, cury);
                        break;
          case '\b':    /* Backspace */
                        handle->GotoXY(--curx, cury);
                        break;
          case '\007':  /* Bell */
                        beep();
                        break;
	  case '\016':	/* SO ; switch to G1 charset */
			handle->WinAttribOff(G0);
			handle->WinAttribOn(G1);
			CharSet = G1;
			break;
	  case '\017':	/* SI ; switch to G0 charset */
			handle->WinAttribOff(G1);
			handle->WinAttribOn(G0);
			CharSet = G0;
			break;
          case '\033':  /* ESC */
                        state = S_ESC;
                        continue;
	  case '\177':	/* Del */
			curx--;
			handle->GotoXY(curx, cury);
			wdelch(handle->Handle);
			break;
          default:      
			if(curx < maxx)
//				handle->Print(c);
				waddch(handle->Handle, c);
 			else
			if(wrap)
                        { if(cury == bot)
                                handle->Scroll(1);
                          else cury++;
                          handle->GotoXY(0, cury);
//			  handle->Print(c);
		   	  waddch(handle->Handle, c);
                        }
                        break;
        }
        break;

      case S_ESC:
	switch(c)
	{ case '[':	state = S_ESC_SQ;
			ResetArgs;
			continue;
	  case '(':	state = S_ESC_RO;
			continue;
	  case ')':	state = S_ESC_RC;
			continue;
	  case 'D':	/* Index , Linefeed */
			if(cury == bot)
				handle->Scroll(1); // ? what if scroll reg 
			else cury++;
			handle->GotoXY(curx, cury);
			break;
	  case 'E':	/* Newline */
			curx = 0;
			if(cury == bot)
				handle->Scroll(1);
			else cury++;
			handle->GotoXY(curx, cury);
			break;
	  case 'H':	/* Horizontal tab set */
			break;
	  case 'M':	/* Reverse Index, Reverse Linefeed */
			if(cury == top)
				handle->Scroll(-1); 
			else cury--;
			handle->GotoXY(curx, cury);
			break;
	  case '7':	/* Save Cursor */
			savex = curx; savey = cury;
			break;
	  case '8':	/* Restore Cursor */
			curx = savex; cury = savey;
			handle->GotoXY(curx, cury);
			break;
	  case 'c':	/* Reset to default */
			handle->WinAttrib(A_NORMAL);
			wsetscrreg(handle->Handle, 0, maxy);
			top = 0; bot = maxy;
			handle->Clear();
			ResetArgs;
			break;
	  case '=':	/* Enter alt keypad mode */
			break;
	  case '>':	/* Exit alt keypad mode */
			break;
	  default:	/* Oops */
//			handle->Printf("^[%c", c);
			state = S_PRINT;
			break;
	}
	state = S_PRINT;
	ResetArgs;
	break;

      case S_ESC_SQ:
        switch(c)
        { case '?':	state = S_ESC_Q;
			ResetArgs;
			continue; 
	  case 'A':     /* Cursor Up */
			if(args[0] == 0)
				cury--;
			else cury -= args[0];
                        handle->GotoXY(curx, cury);
                        break;
          case 'B':     /* Cursor Down */
			if(args[0] == 0)
				cury++;
			else cury += args[0];
                        handle->GotoXY(curx, cury);
                        break;
          case 'C':     /* Cursor Right */
			if(args[0] == 0)
				curx++;
			else curx += args[0];
                        handle->GotoXY(curx, cury);
                        break;
          case 'D':     /* Cursor Left */
			if(args[0] == 0)
				curx--;
			else curx -= args[0];
                        handle->GotoXY(curx, cury);
                        break;
	  case 'f':	/* fall through */
          case 'H':     /* Direct Cursor Address */
			/* Args start from 1 upto max coll/row */
			cury = args[0] ? args[0] - 1 : 0;
			curx = args[1] ? args[1] - 1 : 0;
                        handle->GotoXY(curx, cury);
                        break;
          case 'I':     /* Reverse Line Feed */
			if(cury == top)
				handle->Scroll(-1);
			else cury--;
                        handle->GotoXY(curx, cury);
                        break;
          case 'J':     /* Screen Erasing */
			switch(args[0])
			{ case 0:	/* cur to eos */
					wclrtobot(handle->Handle);
					break;
			  case 1:	/* beg to cur */
					break;
			  case 2:	/* entire screen */
					handle->Clear();
					break;
			}
                        break;
          case 'K':     /* Line Erasing */
			switch(args[0])
 			{ case 0:	/* cur to eol */
					wclrtoeol(handle->Handle);
					break;
			  case 1:	/* beg to cur */
					handle->Printf(0,cury,"%*s", curx, " ");
					break;
			  case 2:	/* line */
					handle->GotoXY(0, cury);
					wclrtoeol(handle->Handle);
					handle->GotoXY(curx, cury);
					break;
			}
                        break;
	  case 'L':	/* Insert Line */
			for(i=0;i<=args[0];i++)
				winsertln(handle->Handle);
			break;
	  case 'M':	/* Delete Line */
			for(i=0;i<=args[0];i++)
				wdeleteln(handle->Handle);
			break;
	  case 'P':	/* Delete character */
			for(i=0;i<=args[0];i++)
				wdelch(handle->Handle);
			break;
	  case 'c':	/* What are you ? */
			out->Put("\033[?l;2c"); /* a vt100 with avo */
			break;
	  case 'g':	/* Tab Clear */
			break;
	  case 'm':	/* Character attibutes */
			for(i=0;i<=argc;i++)
			{ switch(args[i])
			  { case 0:	/* all attrib off */
					handle->WinAttrib(A_NORMAL);
					handle->WinAttribOn(CharSet);
					break;
			    case 1:	/* Bold */
					handle->WinAttribOn(A_BOLD);
					break;
			    case 4:	/* Underline */
					handle->WinAttribOn(A_UNDERLINE);
					break;
			    case 5:	/* Blink */
					handle->WinAttribOn(A_BLINK);
					break;
			    case 7:	/* Reverse */
					handle->WinAttribOn(A_REVERSE);
					break;
			  }
			}
			break;
	  case 'n':	/* Status Reporting */
			switch(args[0])
			{ case 5:	/* Terminal Status */
					out->Put("\0330n");
					break;
			  case 6:	/* Cursor Pos */
					sprintf(wxBuffer, "\033[%d;%dR", cury, curx);
					out->Put(wxBuffer);
					break;
			  case 15:	/* Printer status */
					out->Put("\033[?13n");
					break;
			}
			break;
	  case 'r':	/* Set Scrolling Region */
			/* args start from 1 upto max coll/row */
			top = args[0] - 1;
			bot = args[1] - 1;
			wsetscrreg(handle->Handle, top, bot);
//			handle->GotoXY(0, 0);
			break;
	  case 'x':	/* send terminal parameters */
			break;
	  case '0':
	  case '1':
	  case '2':
	  case '3':
	  case '4':
	  case '5':
	  case '6':
	  case '7':
	  case '8':
	  case '9':
			args[argc] = 10 * args[argc] + c - '0';
			continue;
	  case ';':	/* num seperator */
			argc++;
			continue;
          default:      /* Oops */
//			handle->Printf("^[[%c", c);
                        state = S_PRINT;
                        break;
        }
        state = S_PRINT;
	ResetArgs;
        break;

      case S_ESC_RO:
	switch(c)
	{ case 'A':
	  case 'B':
			G0 = A_NORMAL;
			break;
	  case '0':	/* Special graphics */
			G0 = A_ALTCHARSET;
			break;
	  case '1':
	  case '2':
			break;
	}
	state = S_PRINT;
	ResetArgs;
	break;

      case S_ESC_RC:
	switch(c)
	{ case 'A':
	  case 'B':
			G1 = A_NORMAL;
			break;
	  case '0':
			/* Special graphics */
			G1 = A_ALTCHARSET;
			break;
	  case '1':
	  case '2':
			break;
	}
	state = S_PRINT;
	ResetArgs;
	break;

      case S_ESC_Q:
	switch(c)
	{ case 'h':	/* Set Mode */
			break;
	  case 'l':	/* Reset Mode */
			break;
          case '0':
          case '1':
          case '2':
          case '3':
          case '4':
          case '5':
          case '6':
          case '7':
          case '8':
          case '9':
                        args[argc] = args[argc] * 10 + c - '0';
                        continue;
          case ';':     /* num seperator */
                        argc++;
                        continue;

	}
	state = S_PRINT;
	ResetArgs;
	break;

      default:          /* error; may not occur */
                        state = S_PRINT;
                        break;
    }; /* end state switch */
  }; /* end while */

  win->Refresh();
}

void wxVT100Emu::ParseProcessInput(int curskey, wxTermBuf *out)
{ Bool suc;

  switch(curskey)
  {
    case KEY_UP:
                        suc = out->Put("\033OA");
                        break;
    case KEY_DOWN:
                        suc = out->Put("\033OB");
                        break;
    case KEY_RIGHT:
                        suc = out->Put("\033OC");
                        break;
    case KEY_LEFT:
                        suc = out->Put("\033OD");
                        break;
    case KEY_DC:
			suc = out->Put("\0177");
			break;
    case KEY_BACKSPACE:
                        suc = out->Put(CNTRL('H'));
                        break;
    case KEY_ESC:	suc = out->Put("\033");
			break;
    default:		suc = out->Put((char) curskey);
               		break;
  };
  if(!suc) beep();
}


