#include "wx_vt52.h"
#include "wx_termb.h"
#include "wx_termw.h"
#include "curswin.h"

wxVT52Emu::wxVT52Emu(void) : wxTermEmu()
{
  state = S_PRINT;
}

void wxVT52Emu::Initialize(wxTermBuf *in, wxTermWindow *win, wxTermBuf *out)
{ int cw, ch;
  CursWnd *cwin = win->handle;

  win->GetClientSize(&cw, &ch);
  cwin->SetCursor(CUR_NORM);
  cwin->GotoXY(0, ch - 1);
}

void wxVT52Emu::ParseProcessOutput(wxTermBuf *in, wxTermWindow *win, wxTermBuf *out)
{ int curx, cury, maxx, maxy;
  char c;

  CursWnd *handle = win->handle;
  handle->GetXY(&curx, &cury);
  win->GetClientSize(&maxx, &maxy);
  maxx--; maxy--;

  while(in->Get(&c))
  { switch(state)
    { case S_PRINT:
        switch(c)
        {
	  case '\0':	/* null */
			break;
          case '\r':    /* Carrige return */
                        curx = 0;
                        handle->GotoXY(curx, cury);
                        break;
          case '\n':    /* Newline */
                        if(cury == maxy)
                                handle->Scroll(1);
                        else cury++;
                        handle->GotoXY(curx, cury);
                        break;
          case '\b':    /* Backspace */
                        handle->GotoXY(--curx, cury);
                        break;
          case '\007':  /* Bell */
                        beep();
                        break;
          case '\033':  /* ESC */
                        state = S_ESC;
                        continue;
	  case '\177':	/* Del */
			curx--;
			handle->GotoXY(curx, cury);
			wdelch(handle->Handle);
			break;
          default:      handle->Print(c);
                        break;
        }
        break;
      case S_ESC:
        switch(c)
        { case 'A':     /* Cursor Up */
                        handle->GotoXY(curx, --cury);
                        break;
          case 'B':     /* Cursor Down */
                        handle->GotoXY(curx, ++cury);
                        break;
          case 'C':     /* Cursor Right */
                        handle->GotoXY(++curx, cury);
                        break;
          case 'D':     /* Cursor Left */
                        handle->GotoXY(--curx, cury);
                        break;
          case 'F':     /* Select special graphics char set */
                        handle->WinAttribOn(A_ALTCHARSET);
                        break;
          case 'G':     /* Select US/UK char set */
                        handle->WinAttribOff(A_ALTCHARSET);
                        break;
          case 'H':     /* Cursor Home */
                        handle->GotoXY(0,0);
                        curx = cury = 0;
                        break;
          case 'I':     /* Reverse Line Feed */
			if(cury == 0)
				handle->Scroll(-1);
			else cury--;
                        handle->GotoXY(curx, cury);
                        break;
          case 'J':     /* Erase to end of screen */
                        wclrtobot(handle->Handle);
                        handle->GetXY(&curx, &cury);
                        break;
          case 'K':     /* Erase to end of line */
                        wclrtoeol(handle->Handle);
                        handle->GetXY(&curx, &cury);
                        break;
          case 'Y':     /* Direct cursor address (2 args) */
                        state = S_CABS_1;
                        continue;
                        break;
          case 'Z':     /* Who are you */
                        out->Put("/Z", 2);
                        break;
          case '^':     /* ignored */
          case '-':
          case 'W':
          case 'X':
          case ']':
          case 'V':
          case '=':
          case '>':
                        break;
          default:      handle->Printf("^[%c", c);
                        state = S_PRINT;
                        break;
        }
        state = S_PRINT;
        break;
      case S_CABS_1:    /* Direct cur addr, first arg */
                        state = S_CABS_2;
                        cury = c - ' ';
                        break;
      case S_CABS_2:    /* Direct cur addr, second arg */
                        state = S_PRINT;
                        curx = c - ' ';
                        handle->GotoXY(curx, cury);
                        break;
      default:          /* error; may not occur */
                        state = S_PRINT;
                        break;
    };
  };

  win->Refresh();
}

void wxVT52Emu::ParseProcessInput(int curskey, wxTermBuf *out)
{ Bool suc;

  switch(curskey)
  {
    case KEY_UP:
                        suc = out->Put("\033A", 2);
                        break;
    case KEY_DOWN:
                        suc = out->Put("\033B", 2);
                        break;
    case KEY_RIGHT:
                        suc = out->Put("\033C", 2);
                        break;
    case KEY_LEFT:
                        suc = out->Put("\033D", 2);
                        break;
    case KEY_DC:
    case KEY_BACKSPACE:
                        suc = out->Put(CNTRL('H'));
                        break;
    default:		suc = out->Put((char) curskey);
               		break;
  };
  if(!suc) beep();
}

void wxVT52Emu::Reset(wxTermBuf *in, wxTermWindow *win, wxTermBuf *out)
{
  in->Reset();
  out->Reset();
  win->handle->Clear();
  state = S_PRINT;
  win->Refresh();
}

